import json

from delorean import Delorean
from twisted.internet import defer
from twisted.python import log

import gryphon.data_service.consts as consts
from gryphon.data_service.pollers.request_poller import RequestPoller
import gryphon.data_service.util as util


class OrderbookPoller(RequestPoller):
    @property
    def orderbook_key(self):
        return '%s_orderbook' % self.exchange_name.lower()

    @property
    def heartbeat_key(self):
        return '%s_orderbook_heartbeat' % self.exchange_name.lower()

    @defer.inlineCallbacks
    def start(self):
        binding_key = '%s.orderbook.tinker' % self.exchange_name.lower()
        self.producer = yield util.setup_producer(consts.ORDERBOOK_QUEUE, binding_key)

        super(OrderbookPoller, self).start()

    def get_bids_and_asks(self, raw_orderbook):
        raw_bids = raw_orderbook['bids']
        raw_asks = raw_orderbook['asks']

        return raw_bids, raw_asks

    def parse_orderbook(self, raw_bids, raw_asks):
        bids = []
        asks = []

        for raw_bid in raw_bids:
            bid = self.parse_order(raw_bid)
            bids.append(bid)

        for raw_ask in raw_asks:
            ask = self.parse_order(raw_ask)
            asks.append(ask)

        return bids, asks

    @defer.inlineCallbacks
    def parse_response(self, response):
        raw_bids, raw_asks = self.get_bids_and_asks(response)

        bids, asks = self.parse_orderbook(raw_bids, raw_asks)

        new_orderbook = {
            'timestamp': Delorean().epoch,
            self.exchange_name: {
                'bids': bids,
                'asks': asks,
            }
        }

        current_orderbook_string = yield self.redis.get(self.orderbook_key)

        if not current_orderbook_string:
            current_orderbook_string = ''

        new_orderbook_string = json.dumps(new_orderbook, ensure_ascii=False)

        # TODO: These will never be the same because of the timestamp.
        if current_orderbook_string != new_orderbook_string:
            # There is a new orderbook. Save it and publish it.
            yield self.redis.set(self.orderbook_key, new_orderbook_string)

            if self.producer:
                self.producer.publish_message(new_orderbook_string)
        else:
            log.msg('No Changes on %s' % self.exchange_name)



"""
Several objects in the gryphon-framework use the pattern of initializing their
properties from a dictionary, in which the properties are unpredictable and may not be
present from a dictionary, in which the properties are unpredictable and may not be
present.
"""

class ConfigurableObject(object):
    def configure(self, configuration):
        """
        This function is called to configure the object. It should be a series of calls
        to self.init_configurable(configuration).
        """
        raise NotImplementedError

    def init_configurable(self, configurable_name, configuration):
        """
        Helper function to initialize a single configurable property on the object
        if we find a value set for it in the configuration. If we don't find an
        entry for a configurable name, or find a None value, we don't don't do anything
        (which should mean we stick to the default if you've written the class
        properly).
        """
        if hasattr(self, configurable_name) is not True:
            raise Exception(
                'Configuration error: %s has no attribute %s' % (
                self.__class__.__name__,
                configurable_name,
            ))

        if (configurable_name in configuration
                and configuration[configurable_name] is not None):
            setattr(
                self,
                configurable_name,
                configuration[configurable_name],
            )



import pyximport; pyximport.install()

from gryphon.lib.exchange.quadriga_btc_cad import QuadrigaBTCCADExchange
from gryphon.tests.exceptional.exchange_wrappers.live_orders import LiveOrdersTest


class TestQuadrigaLiveOrders(LiveOrdersTest):
    def __init__(self):
        super(TestQuadrigaLiveOrders, self).__init__()

        # Quadriga has a 10 CAD minimum order size.
        self.order1_price_amount = '10.01'
        self.order2_price_amount = '10.02'

    def setUp(self):
        self.exchange = QuadrigaBTCCADExchange()



import time
import subprocess
import termcolor as tc

from delorean import Delorean
from sqlalchemy import func
from sqlalchemy.orm import joinedload

from gryphon.execution.lib.heartbeat import heartbeat
from gryphon.lib import assets
from gryphon.lib import session
from gryphon.lib.exchange.exchange_factory import ALL_EXCHANGE_KEYS
import gryphon.lib.gryphonfury.positions as positions
import gryphon.lib.gryphonfury.revenue as revenue_lib
from gryphon.lib.logger import get_logger
from gryphon.lib.models.datum import Datum
from gryphon.lib.models.order import Order
from gryphon.lib.models.trade import Trade
from gryphon.lib.money import Money

logger = get_logger(__name__)


DAILY_PROFIT_THRESHOLD = Money.loads('USD -500')
OPEN_PL_THRESHOLD = Money.loads('USD -200')
TICK_SLEEP = 30
FV_MOVEMENT_MINIMUM = 0.10
LARGE_EXCHANGES = [
    'bitfinex',
    'bitstamp',
    'coinbase',
    'itbit',
    'kraken',
    'okcoin'
]
INTER_EXCHANGE_SPREAD_THRESHOLD = 5
PREDICTIVE_VOLUME_REDIS_KEY = 'ML_PREDICTION'

OVERWATCH_HEARTBEAT_KEY = 'OVERWATCH_%s'


def watch():
    db = session.get_a_trading_db_mysql_session()
    r = session.get_a_redis_connection()

    logger.info('Reporting for duty.')

    try:
        while True:
            logger.info('Scanning for abnormalities.')

            check_profit(db)
            check_open_pl(db)
            check_ticktimes(db)
            check_position(db)
            check_btc_net_assets(db)
            check_spreads_are_normal(db)
            check_fv_not_stagnant(db)
            check_fv_predictor_key_set(r)

            # for some reason this commit is needed to pick up changes on
            # subsequent queries.
            session.commit_mysql_session(db)

            logger.info('Going to sleep for %s seconds.' % TICK_SLEEP)
            time.sleep(TICK_SLEEP)
    finally:
        db.remove()


def check_profit(db):
    # beginning of current UTC day
    start_time = Delorean().truncate('day').datetime

    # This profit calculating logic takes about 10s to run
    open_position_offset = positions.fast_position(
        db,
        end_time=start_time,
    )

    open_position_trades = revenue_lib.open_position_trades(
        open_position_offset,
        db,
        start_time,
    )

    trades = db\
        .query(Trade)\
        .join(Order)\
        .filter(Order.actor == 'Multi')\
        .filter(Trade.time_created >= start_time)\
        .options(joinedload('order'))\
        .order_by(Trade.time_created)\
        .all()

    matched_trades, _ = revenue_lib.split_trades(
        open_position_trades + trades,
    )

    daily_pl = revenue_lib.realized_pl(matched_trades, 'USD')

    logger.info('Daily Profit to date: %s' % daily_pl)

    if daily_pl > DAILY_PROFIT_THRESHOLD:
        succeed('PROFIT')
    else:
        fail('PROFIT')


def check_open_pl(db):
    # beginning of current UTC day
    start_time = Delorean().datetime

    # This profit calculating logic takes about 10s to run
    open_position_offset = positions.fast_position(
        db,
        end_time=start_time,
    )

    open_position_trades = revenue_lib.open_position_trades(
        open_position_offset,
        db,
        start_time,
    )

    latest_order = db\
        .query(Order)\
        .order_by(Order.time_created.desc())\
        .filter(Order.actor == 'Multi')\
        .filter(Order._fundamental_value != None)\
        .first()

    open_pl = revenue_lib.open_pl(
        open_position_trades,
        latest_order.fundamental_value.to('USD'),
        'USD',
    )

    logger.info('Open PL: %s' % open_pl)

    if open_pl > OPEN_PL_THRESHOLD:
        succeed('OPEN_PL')
    else:
        fail('OPEN_PL')


# Stop heartbeating if there are any ticktimes longer than a minute in the
# last five minutes.
def check_ticktimes(db):
    five_minutes_ago = Delorean().truncate('minute').last_minute(5).naive

    longest_tick = db\
        .query(Datum)\
        .filter(Datum.time_created >= five_minutes_ago)\
        .filter(Datum.datum_type.like('%TICK_TIME%'))\
        .order_by(Datum.numeric_value.desc())\
        .first()

    if longest_tick:
        logger.info('Longest recent tick: %s' % longest_tick.numeric_value)

        if longest_tick.numeric_value < 60:
            succeed('TICKTIME')
        else:
            fail('TICKTIME')
    else:
        fail('TICKTIME', 'No recent tick data')


def check_position(db):
    cached_position = positions.cached_multi_position(db)
    multi_position = positions.fast_position(db)

    if multi_position == cached_position:
        succeed('POSITION')
    else:
        detail = 'Cached: %s != DB: %s' % (cached_position, multi_position)
        fail('POSITION', detail)


def check_btc_net_assets(db):
    btc_net_assets_error = assets.calculate_btc_net_assets_error(db)

    if btc_net_assets_error == Money(0, 'BTC'):
        succeed('BTC_NET_ASSETS')
    else:
        detail = 'Error: %s' % btc_net_assets_error
        fail('BTC_NET_ASSETS', detail)


def check_spreads_are_normal(db):
    """
    Check whether any exchange has diverged by more than $10 from core fv in the
    last five minutes.
    """

    current_core_fv_datum = db\
        .query(Datum)\
        .filter(Datum.datum_type.like('%CORE_FUNDAMENTAL_VALUE'))\
        .order_by(Datum.time_created.desc())\
        .first()

    current_core_fv = current_core_fv_datum.numeric_value
    last_five = Delorean().last_minute(5).naive
    native_fvs = {}

    for exchange_name in ALL_EXCHANGE_KEYS:
        datum_name = '%s_NATIVE_FUNDAMENTAL_VALUE' % exchange_name.upper()

        if exchange_name in LARGE_EXCHANGES:
            exchange_native_fv = db\
                .query(Datum)\
                .filter(Datum.datum_type == datum_name)\
                .filter(Datum.time_created > last_five)\
                .order_by(Datum.time_created.desc())\
                .first()

            if exchange_native_fv:
                native_fvs[exchange_name] = exchange_native_fv.numeric_value

    sanity = True

    for exchange_name, fv in native_fvs.iteritems():
        if abs(fv - current_core_fv) > INTER_EXCHANGE_SPREAD_THRESHOLD:
            sanity = False
            break

    if sanity:
        succeed('SPREADS')
    else:
        fail('SPREADS')


def check_fv_not_stagnant(db):
    """
    Check that each large exchange's fundamental value is moving normally, catching
    the bug where coinbase had the same FV for hours on end a few weeks ago (18 Oct
    2015).

    We raise an alarm if the fundamental value hasn't moved by more than ten cents in
    the last period. For large exchanges this period is 30 minutes and  for small
    exchanges this period is an hour.
    """

    last_thirty = Delorean().last_minute(30).naive
    sanity = True

    for exchange_name in ALL_EXCHANGE_KEYS:
        datum_name = '%s_NATIVE_FUNDAMENTAL_VALUE' % exchange_name.upper()

        if exchange_name in LARGE_EXCHANGES:
            datums = db\
                .query(
                    func.max(Datum.numeric_value),
                    func.min(Datum.numeric_value),
                )\
                .filter(Datum.datum_type == datum_name)\
                .filter(Datum.time_created > last_thirty)\
                .first()

            max_fv = datums[0]
            min_fv = datums[1]

            if max_fv is None or min_fv is None:
                # Currently have a bug whereby native fv's are not being recorded all the
                # time, this is a hack around that for now.
                continue

            if max_fv - min_fv < FV_MOVEMENT_MINIMUM:
                logger.info(exchange_name + ' fundamental value not moving as expected!')
                sanity = False
                break

    if sanity:
        succeed('FV_MOTION')
    else:
        fail('FV_MOTION')


def check_fv_predictor_key_set(r):
    val = r.get(PREDICTIVE_VOLUME_REDIS_KEY)

    if val is not None and (int(val) == 1 or int(val) == 0):
        succeed('FV_PREDICTOR_KEY')
    else:
        fail('FV_PREDICTOR_KEY')


def succeed(check_id):
    heartbeat(OVERWATCH_HEARTBEAT_KEY % check_id)
    message = '%s passed' % check_id
    logger.info(tc.colored(message, 'green'))


def fail(check_id, detail_msg=None):
    message = '%s failed' % check_id

    if detail_msg:
        message += ': %s' % detail_msg

    logger.error(tc.colored(message, 'red'))




"""
A library that makes it easy to read from .conf files.
"""

import argparse
from collections import defaultdict
import ConfigParser
import StringIO

from cdecimal import Decimal, InvalidOperation

from gryphon.lib.money import Money


def read_config_from_file(config_filename):
    """
    Take a filename in the current working directory and read the configuration therin
    into a dictionary object.

    Section titles of the form [x:y] are parsed into sub-dictionaries under the key x.
    """
    parser = ConfigParser.RawConfigParser()
    parser.read(config_filename)

    configuration = parse_sections(parser)

    return configuration


def _get_parser_for_string_config(string_config):
    """
    This should only be used for unit testing.
    """
    buf = StringIO.StringIO(string_config)
    parser = ConfigParser.ConfigParser()
    parser.readfp(buf)

    return parser


def parse_sections(parser):
    configuration = defaultdict(lambda: {})

    for section_name in parser.sections():
        if ':' in section_name:
            parent_section_name = section_name[:section_name.index(':')]
            subsection_name = section_name[section_name.index(':') + 1:]

            configuration[parent_section_name][subsection_name] = dict(
                parse_section_items(parser.items(section_name)),
            )
        else:
            configuration[section_name] = dict(
                parse_section_items(parser.items(section_name)),
            )

    return configuration


def parse_section_items(items):
    """
    Parse values in the configuration file into object types intelligently.
    """
    parsed_items = []

    for item in items:
        parsed_items.append((item[0], parse_configurable_value(item[1])))

    return parsed_items


def parse_configurable_as_list(value):
    """
    For string values with commas in them, try to parse the value as a list.
    """
    if ',' in value:
        return [token.strip() for token in value.split(',') if token]
    else:
        return [value]


def parse_configurable_value(value):
    """
    Simple function to parse a setting specified either in a .conf file or from the
    command line.
    """
    if value is None:
        return None
    if type(value) is bool:
        return value
    elif value == 'yes':
        return True
    elif value == 'no':
        return False
    else:
        try:
            return Money.loads(value)
        except ValueError:
            try:
                return Decimal(value)
            except InvalidOperation:
                pass

        return value


def dict_update_override(dicta, dictb):
    """
    This
        i) does add None's from b if k is not in a.
        ii) does not override a valid value in a with a None from b
        iii) does override a valid value in a with a valid value from b
    """
    dicta.update(
        {k: v for k, v in dictb.iteritems() if v or (k not in dicta)}
    )




# -*- coding: utf-8 -*-
from collections import defaultdict
import datetime
import logging

from cdecimal import Decimal
from delorean import Delorean
import tornado.web

from gryphon.lib.configurable_object import ConfigurableObject
from gryphon.lib.gryphonfury import positions
from gryphon.lib.gryphonfury import revenue as revenue_lib
from gryphon.lib.models.trade import Trade
from gryphon.lib.money import Money
from gryphon.dashboards.handlers.admin_base import AdminBaseHandler
from gryphon.dashboards.handlers.mixins.start_and_end_time import StartAndEndTimeMixin
from gryphon.dashboards.handlers.strategies.builtin_configs import BUILTIN_STRAT_CONFIGS
from gryphon.dashboards.util import queries

logger = logging.getLogger(__name__)


class StrategyTradingHandler(AdminBaseHandler, StartAndEndTimeMixin, ConfigurableObject):
    def __init__(self, *args, **kwargs):
        super(StrategyTradingHandler, self).__init__(*args, **kwargs)

        # Configurable properties with defaults.
        self.display_name = None
        self.strategy_actor = None
        self.price_currency = None
        self.volume_currency = None
        self.base_point_radius = None
        self.graph_volume_threshold = None
        self.position_graph_max = 4
        self.position_graph_min = -4

    def configure(self, configuration):
        self.init_configurable('display_name', configuration)
        self.init_configurable('strategy_actor', configuration)
        self.init_configurable('price_currency', configuration)
        self.init_configurable('volume_currency', configuration)
        self.init_configurable('base_point_radius', configuration)
        self.init_configurable('graph_volume_threshold', configuration)
        self.init_configurable('position_graph_max', configuration)
        self.init_configurable('position_graph_min', configuration)

    @tornado.web.authenticated
    def get(self, strategy_name):
        strategy_configuration = None

        if strategy_name in self.configuration['strategies']:
            strategy_configuration = self.configuration['strategies'][strategy_name]
        elif strategy_name in BUILTIN_STRAT_CONFIGS.keys():
            strategy_configuration = BUILTIN_STRAT_CONFIGS[strategy_name]
        else:
            self.redirect('/404')
            return

        self.configure(strategy_configuration)

        template_args = self.generate_data()

        self.render_template('strategy.html', args=template_args)

    def generate_data(self):
        start_time, end_time = self.get_start_time_and_end_time()
        start_end_delta = end_time - start_time

        trades, fundamental_value, open_position_trades, open_position_offset = self.get_trades_and_fundamental_value(self.strategy_actor, start_time, end_time)

        trade_data, total_volume, daily_volumes, total_position, position_series = self.basic_trade_analysis(self.strategy_actor, trades, start_end_delta, open_position_offset, start_time)

        matched_trades, position_trades = revenue_lib.split_trades(
            open_position_trades + trades,
            volume_currency=self.volume_currency,
        )

        profit, revenue, matched_fees, matched_volume_currency_fees, open_pl, open_position = self.get_current_trading_result(
            matched_trades,
            position_trades,
            self.price_currency,
            fundamental_value,
        )

        revenue_series = self.get_revenue_series(
            matched_trades,
            start_time,
        )

        template_args = {
            'price_currency': self.price_currency,
            'volume_currency': self.volume_currency,
            'trade_data': trade_data,
            'revenue_series': revenue_series,
            'position_series': position_series,
            'display_name': self.display_name,
            'profit': profit,
            'open_position': open_position,
            'open_pl': open_pl,
            'volume': total_volume,
            'matched_fees': matched_fees,
            'matched_volume_currency_fees': matched_volume_currency_fees,
            'revenue': revenue,
            'start_end_delta': start_end_delta,
            'start_timestamp': Delorean(start_time, "UTC").epoch * 1000,
            'end_timestamp': Delorean(end_time, "UTC").epoch * 1000,
            'now_timestamp': Delorean().epoch * 1000,
            'base_point_radius': float(self.base_point_radius),
            'position_graph_min': float(self.position_graph_min),
            'position_graph_max': float(self.position_graph_max),
        }

        return template_args

    def basic_trade_analysis(self, exchange_name, trades, start_end_delta, open_position_offset, start_time):
        trade_data = {}

        for key in ['core']:
            trade_data[key] = {}

            for side in ['bids', 'asks']:
                trade_data[key][side] = {}
                trade_data[key][side]['prices'] = []
                trade_data[key][side]['volumes'] = []

        total_volume = 0
        daily_volumes = defaultdict(lambda: 0)

        # start from yesterday's open position
        total_position = open_position_offset
        position_series = []

        position_series.append([
            Delorean(start_time, "UTC").epoch * 1000,
            str(open_position_offset.amount),
        ])

        for trade in trades:
            # ignore outliers from rounding errors
            if trade.volume < Money("0.0001", self.volume_currency):
                continue

            timestamp = Delorean(trade.time_created, "UTC").epoch * 1000

            if start_end_delta <= datetime.timedelta(weeks=1):
                # convert timestamp to an hour
                hour = Delorean(trade.time_created, "UTC").truncate('hour').epoch
                daily_volumes[hour] += trade.volume
            else:
                # convert timestamp to a day
                day = Delorean(trade.time_created, "UTC").truncate('day').epoch
                daily_volumes[day] += trade.volume

            total_volume += trade.volume

            price = trade.price_in_currency(self.price_currency)

            unit_price = price / trade.volume.amount

            trade_type = 'core'
            datums = {}

            for datum in trade.order.datums:
                datums[datum.datum_type] = datum

            if trade.volume > self.graph_volume_threshold:
                if trade.trade_type == Trade.BID:
                    side = "bids"
                else:
                    side = "asks"

                trade_data[trade_type][side]['prices'].append([
                    timestamp,
                    str(unit_price.amount),
                    trade.order.exchange.friendly_name,
                    str(trade.volume),
                    str(unit_price),
                    str(trade.fee),
                    trade.order.order_id,
                ])

                trade_data[trade_type][side]['volumes'].append(
                    str(trade.volume.amount)
                )

            total_position += trade.position[self.volume_currency]

            position_series.append([timestamp, str(total_position.amount)])

        return trade_data, total_volume, daily_volumes, total_position, position_series

    def get_revenue_series(self, matched_trades, start_time):
        total_revenue = 0
        revenue_series = []

        profit_units = revenue_lib.profit_units(matched_trades)

        for unit in profit_units:
            timestamp = Delorean(unit['time'], "UTC").epoch * 1000
            total_revenue += unit['revenue']

            revenue_series.append([timestamp, str(total_revenue.amount)])

        return revenue_series

    def get_current_trading_result(self, matched_trades, position_trades, currency, fundamental_value=None):
        profit, revenue, fees, volume_currency_fees = revenue_lib.profit_data(
            matched_trades,
            currency,
            volume_currency=self.volume_currency,
        )

        if fundamental_value:
            open_pl = revenue_lib.open_pl(
                position_trades,
                fundamental_value,
                currency,
                volume_currency=self.volume_currency,
            )
        else:
            open_pl = None

        open_position = positions.position_delta(
            position_trades,
            volume_currency=self.volume_currency,
        )[self.volume_currency]

        return profit, revenue, fees, volume_currency_fees, open_pl, open_position

    def get_trades_and_fundamental_value(self, exchange, start_time, end_time):
        start_time = queries.strip_datetime_for_db_operations(start_time)
        end_time = queries.strip_datetime_for_db_operations(end_time)

        trades = queries.get_strategy_trades_for_period(
            self.trading_db,
            self.strategy_actor,
            start_time,
            end_time,
        )

        latest_order = queries.get_strategy_latest_order(
            self.trading_db,
            self.strategy_actor,
            start_time,
            end_time,
        )

        open_position_offset = Money('0', self.volume_currency)
        open_position_trades = []

        open_position_offset = positions.fast_position(
            self.trading_db,
            end_time=start_time,
            volume_currency=self.volume_currency,
            actor=self.strategy_actor,
        )

        open_position_trades = revenue_lib.open_position_trades(
            open_position_offset,
            self.trading_db,
            start_time,
            volume_currency=self.volume_currency,
            strategy_actor=self.strategy_actor,
        )

        fundamental_value = None

        if latest_order and latest_order.fundamental_value:
            fundamental_value = latest_order.fundamental_value

            if self.price_currency == "USD":
                fundamental_value = fundamental_value.to(
                    "USD",
                    exchange_rate_to_usd=latest_order.exchange_rate,
                )

        return trades, fundamental_value, open_position_trades, open_position_offset




# -*- coding: utf-8 -*-
import logging
import tornado
import tornado.template
import os
from os.path import dirname, abspath
from tornado.options import define, options
from gryphon.lib.logperf import log_request_perf
import uimodules

# Make filepaths relative to settings.
path = lambda root,*a: os.path.join(root, *a)
ROOT = dirname(dirname(abspath(__file__)))


env_var = os.environ

define("port", default=env_var.get('PORT'), help="run on the given port", type=int)
define("config", default=None, help="tornado config file")
define("debug", default=False, help="debug mode")
define("env", default=env_var['APP_ENV'], help='Environment variable')
define('output_file', default='profile.out')
tornado.options.parse_command_line()

APP_ROOT = env_var['APP_ROOT']
MEDIA_ROOT = path(ROOT, APP_ROOT)
TEMPLATE_ROOT = path(ROOT, APP_ROOT)

# Deployment Configuration
class DeploymentType:
    PRODUCTION = "PRODUCTION"
    LOCAL = "LOCAL"
    STAGING = "STAGING"
    dict = {
        PRODUCTION: 1,
        LOCAL: 2,
        STAGING: 3
    }

if options.env:
    DEPLOYMENT = options.env
else:
    DEPLOYMENT = DeploymentType.LOCAL

settings = {}
settings['log_function']            = log_request_perf
settings['deployment']              = DEPLOYMENT
settings['debug']                   = DEPLOYMENT != DeploymentType.PRODUCTION or options.debug 
settings['xsrf_cookies']            = True
settings['template_loader']         = tornado.template.Loader(TEMPLATE_ROOT)
settings['login_url']               = '/login'
settings['cookie_secret']           = env_var['APP_COOKIE_SECRET']
settings["static_path"]             = os.path.join(os.path.dirname(__file__), "static")
settings['ui_modules']              = uimodules



import numpy as np
import time

sampleData = open('sampleData.txt','r').read()
splitData = sampleData.split('\n')

date,closep,highp,lowp,openp,volume = np.loadtxt(splitData,
                                                 delimiter=',',
                                                 unpack=True)

def HHLL(d,c,tf):
    hh = []
    ll = []
    x = tf

    while x < len(d):
        consHigh = c[x-tf:x]
        consLow = c[x-tf:x]

        HighestHigh = max(consHigh)
        LowestLow = min(consLow)

        hh.append(HighestHigh)
        ll.append(LowestLow)

        x+=1
        
    #print hh
    return d,hh,ll
    
HHLL(date,closep,5)




import heroku

class HerokuWorkerScale(object):
    def __init__(self, api_key, app_name):
        self.key = api_key
        self.name = app_name
    
    def _scale(self, num_change):
        if self.key:
            cloud = heroku.from_key(self.key)
            app = cloud.apps[self.name]
            workers = app.processes['worker']
            len_workers = len([worker for worker in workers])
            workers.scale(len_workers+num_change)
    def add_worker(self):
        self._scale(1)
    
    def remove_worker(self):
        self._scale(-1)
        



# -*- coding: utf-8 -*-
import requests
import os

try:
    PAPERTRAIL_TOKEN = os.environ['PAPERTRAIL_API_TOKEN']
except:
    raise Exception("""Requires environment variables: PAPERTRAIL_API_TOKEN""")

class Papertrail(object):
    @staticmethod
    def get_logs(query):
        """
        curl -v -H "X-Papertrail-Token: abc123" "https://papertrailapp.com/api/v1/events/search.json?q='Critical error'"
        """
        search_query = "%s" % query
        url = 'https://papertrailapp.com/api/v1/events/search.json?q=%s' % search_query
        headers = {'X-Papertrail-Token':PAPERTRAIL_TOKEN}
        r = requests.get(url, headers=headers)
        if r.status_code == 200:
            return r.json()
        else:
            return {}




"""
This class re-implements the common methods of the ExchangeAPIWrapper class in a way
that also records the activity in a mysql database.

It also provides some other functions that can be useful for tracking activity on the
exchange account.

Currently this does not support the exchange wrapper's use of request-futures, but it
shouldn't be too hard to do so. You simply do most of the db work in the resp function.

To the strategies, this should appear to be an ExchangeAPIWrapper, but under the hood
it makes sure that our database does not get out of sync with our our account with the
exchange.
"""

import pyximport; pyximport.install()

from sets import Set

from cdecimal import Decimal, ROUND_UP, ROUND_DOWN
from delorean import Delorean
from sqlalchemy.orm import joinedload

from gryphon.execution.lib import auditing
from gryphon.lib.exchange import exceptions
from gryphon.lib.exchange import order_types
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.exchange_api_wrapper import ExchangeAPIWrapper
from gryphon.lib.exchange.retry import exchange_retry
from gryphon.lib.models.datum import Datum
from gryphon.lib.models.event import EventRecorder
from gryphon.lib.models.order import Order
from gryphon.lib.models.exchange import Exchange as ExchangeData
from gryphon.lib.money import Money
from gryphon.lib.session import commit_mysql_session
from gryphon.lib.util.profile import tick_profile


class ExchangeCoordinator(object):
    def __init__(self, exchange_wrapper, db, harness=None):
        self.db = db
        self.exchange_wrapper = exchange_wrapper
        self.harness = harness
        self.is_tradable = False

        # Coordinators can only be used for trading if they are connected to an
        # initialized ledger in the data store, but they can be used for public
        # exchange API functions like get_orderbook().
        try:
            self.exchange_account = self.exchange_wrapper.exchange_account_db_object(db)
            self.is_tradable = True
        except AssertionError:
            self.exchange_account = None

        # TBD whether this has any purpose here. Execute/no-execute should be handled
        # in harness.pyx I think, but it still might be useful to have a dry-run mode
        # in ExchangeCoordinator for tests maybe.
        # self.execute = False

        # This represents whether this exchange has had any trading activity on it
        # since the last audit. If not, there's no reason to run consolidate_ledger()
        # or audit() for this exchange. Potentially, we could make this even less
        # strict, setting this to False whenever there are no orders left on the books.
        self.is_active = False
  
        """
        This actually means that we won't audit in no-execute mode.
        We do want audits to take place even if nothing has happened... hmm...
        """

    def consolidate_ledger(self):
        """
        To subsume harness::run_accounting
        """
        open_orders = self.get_open_orders()
        eaten_order_ids, current_orders = self._get_current_orders(open_orders)
        self._run_accounting(eaten_order_ids, current_orders)

        return current_orders

    @tick_profile
    def _get_current_orders(self, exchange_open_orders):
        db_open_orders = self._get_db_open_orders()
        db_open_order_ids = Set([o.exchange_order_id for o in db_open_orders])

        exchange_open_order_ids = Set([o['id'] for o in exchange_open_orders])
        eaten_order_ids = db_open_order_ids - exchange_open_order_ids
        current_order_ids = db_open_order_ids & exchange_open_order_ids

        unexpected_order_ids = exchange_open_order_ids - db_open_order_ids

        if unexpected_order_ids:
            self.handle_unexpected_orders(unexpected_order_ids)

        current_orders = []

        for order in exchange_open_orders:
            if order['id'] in current_order_ids:
                current_orders.append(order)

        return eaten_order_ids, current_orders

    def _run_accounting(self, eaten_order_ids, current_orders):
        """
        This function runs before each tick and guarantees that our ledger is consistent
        with the exchange's state as we enter the tick body.

        eaten_order_ids are the exchange order ids that have disappeared form the open
            state on the exchange since the last tick.

        current_orders are the entries from the response to exchange.open_orders that
            aren't in unexpected_order_ids.

        This function does something very similar in each case.
            - get the exchange details for the order
            - get the db order
            - run was_eaten/was_partially_eaten, save the order, and update
              the exchange_account position

        The only difference is for current orders we only do the last step if their
        volume_remaining has changed.
        """

        if eaten_order_ids:
            eaten_order_details = self.get_multi_order_details(eaten_order_ids)
            order_ids = eaten_order_details.keys()
            orders = self._get_orders_by_order_ids(order_ids)

            for order in orders:
                order_data = eaten_order_details[order.exchange_order_id]

                position_change, position_change_no_fees = order.was_eaten(order_data)
                self._save_order(order)
                self.update_position(position_change, position_change_no_fees)

                self.harness.log_trade(
                    position_change_no_fees[self.exchange_wrapper.currency],
                    position_change_no_fees[self.exchange_wrapper.volume_currency],
                )

        for current_order_from_exchange in current_orders:
            current_order_from_db = self._get_orders_by_order_ids(
                [current_order_from_exchange['id']],
            )[0]

            # Partially filled.
            if (current_order_from_exchange['volume_remaining'] 
                    < current_order_from_db.volume_remaining):

                details_result = self.get_multi_order_details(
                    [current_order_from_exchange['id']],
                )

                current_order_details = details_result[current_order_from_exchange['id']]

                position_change, position_change_no_fees = current_order_from_db.was_partially_eaten(current_order_details)

                self._save_order(current_order_from_db)
                self.update_position(position_change, position_change_no_fees)

                self.harness.log_trade(
                    position_change_no_fees[self.exchange_wrapper.currency],
                    position_change_no_fees[self.exchange_wrapper.volume_currency],
                )

    def handle_unexpected_orders(self, unexpected_order_ids):
        """
        Any orders that we find are open on the exchange account, but according to
        our ledger shouldn't be, are handled in this function. It considers three cases,
        documented in comments.
        """
        db_unexpected_orders = self._get_orders_by_order_ids(unexpected_order_ids)
        db_unexpected_order_ids = [o.exchange_order_id for o in db_unexpected_orders]

        # Case one, there are open orders on the exchange that are nowhere to be found
        # in our database.
        if set(db_unexpected_order_ids) != set(unexpected_order_ids):
            raise auditing.AuditException(
                'Unexpected Orders (not in db): %s' % unexpected_order_ids,
            )

        # Case two, there are orders open on the exchange that are neither open nor
        # cancelled in our database.
        real_unexpected_orders = [
            o for o in db_unexpected_orders if o.status != 'CANCELLED'
        ]

        if real_unexpected_orders:
            raise auditing.AuditException(
                'Unexpected Orders (shouldn\'t be in open state on exchange): %s' % [
                o.exchange_order_id for o in real_unexpected_orders
            ])

        # Case three, orders that are open on the exchange but are cancelled in our
        # database. This is probably just a past request failure, so we just try to
        # re-cancel them and let the next iteration of sync_ledger re-examine them.
        should_have_been_cancelled_orders = [
            o for o in db_unexpected_orders if o.status == 'CANCELLED'
        ]

        for o in should_have_been_cancelled_orders:
            #self.harness.log(
            #    '#%s should have been cancelled. Trying again.', o.exchange_order_id,
            #)

            o.status = 'OPEN'

            self.db.add(o)

            self.cancel_order(o.exchange_order_id)

    def _get_orders_by_order_ids(self, order_ids):
        return self.db.query(Order)\
            .filter_by(_exchange_name=self.exchange_wrapper.name)\
            .filter(Order.exchange_order_id.in_(order_ids))\
            .all()

    def _get_db_open_orders(self):
        return self.db.query(Order)\
            .filter_by(_exchange_name=self.exchange_wrapper.name)\
            .filter_by(status=Order.OPEN)\
            .options(joinedload('datums'))\
            .all()

    def update_position(self, position_change, position_change_no_fees):
        """
        Formerly harness:update_position.
        """ 
        for currency_code, position in position_change.iteritems():
            self.exchange_account.position[currency_code] += position
            self.exchange_account.balance[currency_code] += position

        self.db.add(self.exchange_account)
        commit_mysql_session(self.db)

    ## Trading interface functions. ##

    def limit_order(self, mode, volume, price, extra_data=[]):
        return self.place_order(
            mode,
            volume,
            price,
            order_types.LIMIT_ORDER,
            extra_data,
        )

    def market_order(self, mode, volume, price, extra_data=[]):
        return self.place_order(
            mode,
            volume,
            price,
            order_types.MARKET_ORDER,
            extra_data,
        )

    def place_order(self, mode, volume, price=None, order_type=order_types.LIMIT_ORDER, extra_data=[]):
        # TODO: this constant should be moved into the exchange wrapper library.
        # TODO: Give some messaging here why the order isn't being placed.
        if volume <= self.exchange_wrapper.min_order_size:
            return

        actor = Order.NULL_ACTOR

        if self.harness is not None:
            actor =  self.harness.strategy.actor

        # Some exchanges only let you place orders to price precision less than a cent.
        # In this case we round the order price in the most conservative direction.
        if self.exchange_wrapper.price_decimal_precision != None:
            if mode == Consts.BID:
                price = price.round_to_decimal_places(
                    self.exchange_wrapper.price_decimal_precision,
                    ROUND_DOWN,
                )
            elif mode == Consts.ASK:
                price = price.round_to_decimal_places(
                    self.exchange_wrapper.price_decimal_precision,
                    ROUND_UP,
                )
            else:
                raise Exception('Order type must be one of Bid or Ask!')

        # Some exchanges also have volume precision that is less than the native
        # precision of that currency.
        if self.exchange_wrapper.volume_decimal_precision != None:
            volume = volume.round_to_decimal_places(
                self.exchange_wrapper.volume_decimal_precision,
                ROUND_DOWN,
            )

        self.harness.log(
            'Placing order: %s for %s %.4f at %s',
            (mode, volume.currency, volume.round_to_decimal_places(4), price),
            color='red',
        )

        if self.harness.execute:
            self.is_active = True

            new_order = self._place_order_on_exchange(mode, volume, price, order_type)

            order = Order(
                actor,
                mode,
                volume,
                price,
                self.exchange_wrapper,
                new_order['order_id'],
            )

            # TODO: Turn the FV column into a datum that market making strategies add.
            order.fundamental_value = Money('1', 'USD')

            order.exchange_rate = Money(
                '1',
                self.exchange_wrapper.currency,
            ).to('USD').amount
  
            if extra_data:
                order.datums = self._create_datums_from_extra_data(extra_data)

            self._save_order(order)

            return new_order
        else:  # no-execute.
            self.harness.log('Not placing order because execute=False')

    def _create_datums_from_extra_data(self, extra_data):
        """
        Accepts a dictionary of key-value pairs, where the keys are strings and the
        values are either strings or Decimals, and returns a list of Datum objects with
        equivalent data.
        """
        datums = []

        for datum_type, value in extra_data.items():
            if type(value) is str:
                datums.append(Datum(datum_type, string_value=value))
            elif type(value) is Decimal:
                datums.append(Datum(datum_type, numeric_value=value))
            elif type(value) is Money:
                datums.append(Datum(
                    datum_type,
                    numeric_value=value.amount,
                    string_value=value.currency,
                ))
            else:
                raise Exception(
                    'extra_data only supports str or cdecimal.Decimal types',
                )

        return datums

    @exchange_retry()
    def _place_order_on_exchange(self, mode, volume, price, order_type=order_types.LIMIT_ORDER):
        new_order = self.exchange_wrapper.place_order(mode, volume, price, order_type)
        return new_order

    def _save_order(self, new_order):
        self.db.add(new_order)
        commit_mysql_session(self.db)

    @tick_profile
    @exchange_retry()
    def get_open_orders(self):
        return self.exchange_wrapper.get_open_orders()

    @exchange_retry()
    def cancel_order(self, order_id):
        try:
            self.exchange_wrapper.cancel_order(order_id)
        except exceptions.CancelOrderNotFoundError:
            # Don't place a new order if the order to cancel got eaten.
            pass

    def cancel_all_open_orders(self):
        # Won't this happen anyway with the getattr hack?
        self.exchange_wrapper.cancel_all_open_orders()

    @tick_profile
    @exchange_retry()
    def get_orderbook(self, open_orders=None):
        orderbook = self.exchange_wrapper.get_orderbook()

        if open_orders is not None:
            orderbook = self.exchange_wrapper.remove_orders_from_orderbook(
                orderbook,
                open_orders,
            )

        return orderbook

    @exchange_retry()
    def get_multi_order_details(self, order_ids):
        return self.exchange_wrapper.get_multi_order_details(order_ids)

    def __getattr__(self, name, *args):
        """
        Right now this class is a weird compromise between composition and inheritance
        with the ExchangeAPIWrapper class. In some senses this class 'has' an api
        wrapper, and in other senses it 'extends' the APIWrapper interface. An
        inheritance solution would have a ton of helper functions in this class that
        just dispatch to self.exchange_wrapper, which seems silly. On the other hand,
        we really do want all the methods on self.exchange_wrapper to be callable
        through this class. This function is a hack to ignore this conundrum for a
        little while, but likely comes with it's own dangers.

        It should be noted that the current method breaks the usage of the req/resp-
        style methods, as those will go straight to the exchange wrapper, but we can
        fix that later.
        """
        # Even better than this check would be `if name in ExchangeTradingInterface`
        if name in dir(self.exchange_wrapper):
            return self.exchange_wrapper.__getattribute__(name, *args)
        else:
            raise AttributeError(
                '\'%s\' object has no attribute \'%s\'' % (
                self.__class__.__name__,
                name,
            ))

    # Audits. #
    def audit(self, audit_types):
        self.exchange_wrapper.pre_audit(exchange_data=self.exchange_account)
        volume_tolerance = self.exchange_wrapper.volume_balance_tolerance
        fiat_tolerance = self.exchange_wrapper.fiat_balance_tolerance

        event_data = {}

        if auditing.ORDER_AUDIT in audit_types:
            order_audit_data = self.order_audit()

            if order_audit_data:
                event_data['order_data'] = order_audit_data

        balance_audit_data = None

        if auditing.VOLUME_BALANCE_AUDIT in audit_types:
            balance_audit_data = self.volume_balance_audit(volume_tolerance)

            if balance_audit_data:
                event_data['balance_data'] = balance_audit_data

        if auditing.FIAT_BALANCE_AUDIT in audit_types:
            # This returns the exact same balance_audit_data as
            # volume_balance_audit() so we don't need to record it here. We can also
            # skip a second balance() call to the exchange by passing in the result
            # of volume_balance_audit() if we've already called that.
            self.fiat_balance_audit(fiat_tolerance, exchange_balance=balance_audit_data)

        if auditing.POSITION_CACHE_AUDIT in audit_types:
            auditing.position_cache_audit(self.db, self.exchange_account)

        if auditing.LEDGER_AUDIT in audit_types:
            auditing.ledger_audit(self.exchange_account)

        EventRecorder().record(Consts.AUDIT, self.name, event_data)

        self.is_active = False

        self.harness.log('Audits Passed', color='green')

    @exchange_retry()
    def volume_balance_audit(self, tolerance, exchange_balance=None):
        self.harness.log(
            'volume_balance_audit(tolerance=%s)' % tolerance,
            log_level='debug',
        )

        return auditing.volume_balance_audit(
            self.exchange_wrapper,
            self.exchange_account,
            tolerance=tolerance,
            execute=self.harness.execute,
            exchange_balance=exchange_balance,
        )

    @exchange_retry()
    def fiat_balance_audit(self, tolerance, exchange_balance=None):
        self.harness.log(
            'fiat_balance_audit(tolerance=%s)' % tolerance,
            log_level='debug',
        )

        return auditing.fiat_balance_audit(
            self.exchange_wrapper,
            self.exchange_account,
            tolerance=tolerance,
            execute=self.harness.execute,
            exchange_balance=exchange_balance,
        )

    @exchange_retry()
    def order_audit(self):
        self.harness.log('order_audit()', log_level='debug')

        try:
            order_audit_data = auditing.order_audit(self.db, self.exchange_wrapper)
        except auditing.OrderAuditException as e:
            # These are all the orders which failed the order audit.
            failed_order_ids = [d[0] for d in e.failed_order_data]

            # We give the exchange a 2nd chance, hoping that things have settled down
            # and we will get the correct accounting this time. If we try to reopen an
            # order that probably shouldn't be, this will cause a failure and kill the
            # bot.
            self.reopen_orders(e.exchange, failed_order_ids)

            # There aren't any open orders because we're in an audit.
            eaten_order_ids, current_orders = self._get_current_orders([])  
            self._run_accounting(eaten_order_ids, current_orders)

            # Now we retry the original audit. If it fails again this will cause a
            # hard failure and kill the bot.
            order_audit_data = auditing.order_audit(self.db, self.exchange_wrapper)

        return order_audit_data

    def reopen_orders(self, exchange, exchange_order_ids):
        for exchange_order_id in exchange_order_ids:
            self.log(
                'Order Volume Mismatch on %s! Attempting to fix.',
                (exchange_order_id),
                'red',
            )

            order = self.db.query(Order)\
                .filter(Order._exchange_name == exchange.name)\
                .filter(Order.exchange_order_id == exchange_order_id)\
                .one()

            if (Delorean(order.time_created, 'UTC').datetime <
                    Delorean().last_hour(1).datetime):
                raise Exception(
                    'Trying to re-open an order that\'s more than an hour old!'
                )

            order.status = Order.OPEN




import cdecimal
import functools
import simplejson as json
from cdecimal import Decimal
from delorean import Delorean
from gryphon.lib import gryphon_json_serialize
from gryphon.lib.logger import get_logger
from gryphon.lib.session import get_a_redis_connection

logger = get_logger(__name__)

"""
    A decorator which stores the result of the function in a redis database

    This is specifically to be used for storing the result of cpu intensive
    calculations which will not change any time in the future.

    This only works with functions on a handler object. We can exted it
    later to work with library and util functions.

    Be *very* careful not to use this on any functions whose results will
    change over time, e.g. calculation revenues for the present month.

    An example usage is investor.py:get_total_vllume_for_exchange_in_period

    You can force the handlers to recalculate any calculations by passing
    &recalculate in the query string.
"""
def store_calculation_result(function):
    @functools.wraps(function)
    def wrapper(*args, **kwargs):
        handler = args[0]

        # remove the 'self' argument on the handlers
        key_args = args[1:]

        # keys in redis are a stringifying of the function name and
        # arguments.
        key = '%s:%s:%s' % (
            function.__name__, 
            str(key_args), 
            str(kwargs),
        )
  
        redis = redis = get_a_redis_connection()

        if not handler.should_recalculate_stored_results:
            redis_result = redis.get(key)

            if redis_result:    
                return parse_value_from_redis(redis_result)

        calculated_value = function(*args, **kwargs)
  
        redis.set(key, prepare_value_for_redis(calculated_value))

        return calculated_value
    return wrapper


def prepare_value_for_redis(value):
    return json.dumps(value, use_decimal=True)


def parse_value_from_redis(value):
    return json.loads(value, parse_float=Decimal)





def skipped(func):
    from nose import SkipTest
    def decorator(*args, **kwargs):
        raise SkipTest("Test %s is skipped" % func.__name__)
    decorator.__name__ = func.__name__
    return decorator



###############################################################################
##
##  Copyright (C) 2011-2013 Tavendo GmbH
##
##  Licensed under the Apache License, Version 2.0 (the "License");
##  you may not use this file except in compliance with the License.
##  You may obtain a copy of the License at
##
##      http://www.apache.org/licenses/LICENSE-2.0
##
##  Unless required by applicable law or agreed to in writing, software
##  distributed under the License is distributed on an "AS IS" BASIS,
##  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
##  See the License for the specific language governing permissions and
##  limitations under the License.
##
###############################################################################

import sys
from optparse import OptionParser
import json
from twisted.python import log
from twisted.internet import reactor, ssl

from autobahn.twisted.websocket import WebSocketClientFactory, \
                                       WebSocketClientProtocol, \
                                       connectWS



class EchoClientProtocol(WebSocketClientProtocol):

    def sendHello(self):
        data = {
            "type": "subscribe",
            "product_id": "BTC-USD"
        }

        self.sendMessage(json.dumps(data).encode('utf8'))

    def onClose(self, wasClean, code, reason):
        log.msg("WebSocket connection closed: %s : Code %s : wasClean:%s. Restarting." % (reason, code, wasClean))

    def onOpen(self):
      self.sendHello()

    def onMessage(self, payload, isBinary):
      if not isBinary:
         print("Text message received: {}".format(payload.decode('utf8')))
      reactor.callLater(1, self.sendHello)



if __name__ == '__main__':

   log.startLogging(sys.stdout)

   parser = OptionParser()
   parser.add_option("-u", "--url", dest = "url", help = "The WebSocket URL", default = "wss://ws-feed.exchange.coinbase.com")
   (options, args) = parser.parse_args()

   ## create a WS server factory with our protocol
   ##
   factory = WebSocketClientFactory(options.url, debug = True)
   factory.protocol = EchoClientProtocol

   ## SSL client context: default
   ##
   if factory.isSecure:
      contextFactory = ssl.ClientContextFactory()
   else:
      contextFactory = None

   connectWS(factory, contextFactory)
   reactor.run()



import os

from twisted.internet import defer
from twisted.internet import protocol
from twisted.internet import reactor
from txredis.client import RedisClient

import gryphon.data_service.consts as consts
from gryphon.data_service.twisted_producer import TwistedProducer


def setup_redis(host='localhost', port=6379):
    """Setup a twisted txredis client. Returns a Deferred"""

    clientCreator = protocol.ClientCreator(reactor, RedisClient)

    return clientCreator.connectTCP(host, port)


@defer.inlineCallbacks
def setup_producer(queue, binding_key):
    if 'AMPQ_HOST' in os.environ:
        producer = TwistedProducer(
            consts.EXCHANGE,
            consts.EXCHANGE_TYPE,
            queue,
            binding_key,
        )

        yield producer.run()

        # Returns defered.
        defer.returnValue(producer)
    else:
        raise ValueError('AMPQ_HOST Must be set in Environment')



import numpy as np
import time

sampleData = open('sampleData.txt','r').read()
splitData = sampleData.split('\n')

date,closep,highp,lowp,openp,volume = np.loadtxt(splitData,
                                                 delimiter=',',
                                                 unpack=True)



def ExpMovingAverage(values, window):
    weights = np.exp(np.linspace(-1., 0., window))
    weights /= weights.sum()
    a =  np.convolve(values, weights, mode='full')[:len(values)]
    a[:window] = a[window]
    return a


def percentChange(startPoint,currentPoint):
    return((float(currentPoint)-startPoint)/abs(startPoint))*100.00


def chaikinVolCalc(emaUsed,periodsAgo):

    chaikin_volatility = []
    highMlow = []
    x = 0
    while x < len(date):
        hml = highp[x]-lowp[x]
        highMlow.append(hml)
        x += 1

    print len(date)
    print len(highMlow)
    highMlowEMA = ExpMovingAverage(highMlow,emaUsed)
    print len(highMlowEMA)
    y = emaUsed + periodsAgo

    while y < len(date):
        cvc = percentChange(highMlow[y-periodsAgo],highMlow[y])
        chaikin_volatility.append(cvc)
        y+=1

    print len(date[emaUsed+periodsAgo:])
    
    return date[emaUsed+periodsAgo:], chaikin_volatility
        


chaikinVolCalc(10,10)



# -*- coding: utf-8 -*-
from collections import defaultdict
from datetime import datetime, date, timedelta
import json
import os
import requests
import uuid

from decimal import *
from delorean import Delorean
from sqlalchemy import ForeignKey, Column, Integer, Unicode, DateTime, UnicodeText, Numeric
from sqlalchemy.orm import relationship, backref
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.ext.hybrid import hybrid_property
from sqlalchemy import and_, func

from gryphon.lib import gryphon_json_serialize
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.exchange_factory import make_exchange_from_key
from gryphon.lib.logger import get_logger
from gryphon.lib.models.base import Base
from gryphon.lib.money import Money
from gryphon.lib.util.list import flatten

logger = get_logger(__name__)

metadata = Base.metadata


class Transaction(Base):
    __tablename__ = 'transaction'
    
    #Transaction Types
    WITHDRAWL = 'WITHDRAWL'
    DEPOSIT = 'DEPOSIT'
    
    #Transaction Status'
    IN_TRANSIT = 'IN_TRANSIT'
    COMPLETED = 'COMPLETED'
    CANCELED = 'CANCELED'
    
    transaction_id = Column(Integer, primary_key=True)
    transaction_type = Column(Unicode(64))
    transaction_status = Column(Unicode(64))
    unique_id = Column(Unicode(64), nullable=False)
    time_created = Column(DateTime, nullable=False)
    time_completed = Column(DateTime, nullable=True)
    
    _amount = Column('amount', Numeric(precision=24, scale=14))
    _amount_currency = Column('amount_currency', Unicode(3))
    _fee = Column('fee', Numeric(precision=24, scale=14))
    _fee_currency = Column('fee_currency', Unicode(3))
    _transaction_details = Column('transaction_details', UnicodeText(length=2**31))
    
    exchange_id = Column(Integer, ForeignKey('exchange.exchange_id'))

    # Some Transactions have BTC fees, which reduce our total BTC assets. Every once in a while
    # we create a "buyback" transaction where we buy back those fees outside our trading system.
    # We then mark the fee as "bought back" by setting the transaction's fee_buyback_transaction
    # This one is a bit complex because it is self-referential
    fee_buyback_transaction_id = Column(Integer, ForeignKey('transaction.transaction_id'))
    fee_buyback_transaction = relationship("Transaction", remote_side=[transaction_id], backref='fee_buyback_transactions')
    
    def __init__(self, transaction_type, transaction_status, amount, exchange, transaction_details, fee=None):
        self.unique_id = unicode(uuid.uuid4().hex)
        self.time_created = datetime.utcnow()
        self.transaction_type = transaction_type
        self.transaction_status = transaction_status
        self.amount = amount
        self.exchange = exchange
        self.transaction_details = transaction_details
        if fee:
            self.fee = fee
        
    def __unicode__(self):
        return u'[TRANSACTION:%s, EXCHANGE:%s, STATUS:%s] Amount:%s (%s) at %s' % (
            self.transaction_type, self.exchange.name, self.transaction_status, self.amount, self.fee, self.time_created)
        
    def __repr__(self):
        return self.to_json()
    
    def to_json(self):
        return json.dumps({
            'transaction_id':self.transaction_id,
            'transaction_type':self.transaction_type,
            'transaction_status':self.transaction_status,
            'time_created':unicode(self.time_created),
            'unique_id':self.unique_id,
            'exchange':self.exchange.name,
            'amount':self.amount,
            'fee': self.fee,
            'transaction_details':self.transaction_details
        }, ensure_ascii=False)
    
    @property
    def transaction_details(self):
        return json.loads(self._transaction_details)

    @transaction_details.setter
    def transaction_details(self, value):
        self._transaction_details = json.dumps(value, ensure_ascii=False)
    
    @property
    def amount(self):
        return Money(self._amount, self._amount_currency)

    @amount.setter
    def amount(self, value):
        self._amount = value.amount
        self._amount_currency = value.currency

    @property
    def fee(self):
        if self._fee and self._fee_currency:
            return Money(self._fee, self._fee_currency)
        else:
            return None

    @fee.setter
    def fee(self, value):
        self._fee = value.amount
        self._fee_currency = value.currency

    @hybrid_property
    def has_outstanding_btc_fee(self):
        return self._fee_currency == 'BTC' and self.fee_buyback_transaction_id == None and self.transaction_status != self.CANCELED

    @has_outstanding_btc_fee.expression
    def has_outstanding_btc_fee(cls):
        return and_(cls._fee_currency == 'BTC', cls.fee_buyback_transaction_id == None, cls.transaction_status != cls.CANCELED)

    def complete(self):
        if self.transaction_status != self.IN_TRANSIT:
            raise ValueError("We can only complete an IN_TRANSIT transaction")
        self.transaction_status = self.COMPLETED
        self.time_completed = datetime.utcnow()

        if self.transaction_type == self.DEPOSIT:
            self.exchange.balance[self.amount.currency] += self.amount
        elif self.transaction_type == self.WITHDRAWL:
            self.exchange.balance[self.amount.currency] -= self.amount

        if self.fee:
            self.exchange.balance[self.fee.currency] -= self.fee

    def cancel(self):
        if self.transaction_status == self.IN_TRANSIT:
            self.transaction_status = self.CANCELED
        elif self.transaction_status == self.COMPLETED:
            self.transaction_status = self.CANCELED

            if self.transaction_type == self.DEPOSIT:
                self.exchange.balance[self.amount.currency] -= self.amount
            elif self.transaction_type == self.WITHDRAWL:
                self.exchange.balance[self.amount.currency] += self.amount

            if self.fee:
                self.exchange.balance[self.fee.currency] += self.fee
        else:
            raise ValueError("We can only cancel an IN_TRANSIT or COMPLETED transaction")

    def confirmations(self):
        if self.transaction_details and 'transaction_hash' in self.transaction_details:
            r = requests.get('https://api.blockcypher.com/v1/btc/main/txs/%s' % self.transaction_details['transaction_hash'])
            response = r.json()
            if 'confirmations' in response:
                return int(response['confirmations'])

        return None

    def already_has_tx_hash(self):
        return (self.transaction_details and
            'transaction_hash' in self.transaction_details and
            self.transaction_details['transaction_hash'])

    def can_lookup_tx_hash(self):
        return (self.amount.currency == "BTC" and
            self.transaction_details and
            'deposit_address' in self.transaction_details)

    def update_tx_hash(self):
        if not self.already_has_tx_hash() and self.can_lookup_tx_hash():
            tx_hash = self.find_on_blockchain()
            logger.info("Found %s %s on the blockchain: %s" % (self.exchange.name, self.amount, tx_hash))

            # can't directly update self.transaction_details because it is a json magic field
            tx_details = self.transaction_details
            tx_details['transaction_hash'] = tx_hash
            self.transaction_details = tx_details

    def find_on_blockchain(self):
        deposit_address = self.transaction_details['deposit_address']
        satoshi_amount = int(self.amount * 10**8)

        r = requests.get('https://api.blockcypher.com/v1/btc/main/addrs/%s/full' % deposit_address)
        response = r.json()
        transactions = response['txs']

        for transaction in transactions:
            for output in transaction['outputs']:
                if deposit_address in output['addresses'] and output['value'] == satoshi_amount:
                    return transaction['hash']

        return None

    # a transaction is stuck if it has been in transit for more than 3 hours
    def is_stuck(self):
        return self.amount.currency == "BTC" and \
            self.transaction_status == self.IN_TRANSIT and \
            Delorean(self.time_created, "UTC") < Delorean().last_hour(3)

    @property
    def position(self):
        from gryphon.lib.models.exchange import Position
        position = Position()
        if self.transaction_type == self.DEPOSIT:
            position[self.amount.currency] += self.amount
        elif self.transaction_type == self.WITHDRAWL:
            position[self.amount.currency] -= self.amount
        if self.fee:
            position[self.fee.currency] -= self.fee
        return position

    @hybrid_property
    def magic_time_completed(self):
        """
        Regular property you can call on a loaded trade in python. Returns a USD Money object
        """
        if self.time_completed:
            return self.time_completed
        else:
            return self.time_created

    @magic_time_completed.expression
    def magic_time_completed(cls):
        """
        SQL Expression which lets us calculate and use USD prices in SQL.

        Must have Order joined in queries where it is used.
        Ex: db.query(func.sum(Trade.price_in_usd)).join(Order).scalar()
        This gives decimal results, since we don't have our Money objects in SQL
        """
        return func.IF(cls.time_completed, cls.time_completed, cls.time_created)



# -*- coding: utf-8 -*-
import logging

from gryphon.dashboards.handlers.admin_base import AdminBaseHandler
from gryphon.dashboards.models.user import User

logger = logging.getLogger(__name__)


class LoginHandler(AdminBaseHandler):
    def get(self):
        if self.get_current_user():
            self.redirect('/')
        else:
            self.render_template('login.html', args={})

    def post(self):
        username = self.get_argument('username')
        password = self.get_argument('password')

        next = self.get_argument('next', '/')

        user = self.dashboard_db.query(User).filter_by(username=username).first()

        if user:
            if user.password == password:
                self.login_user(user)
                self.redirect(next)
            else:
                self.show_error_message('Incorrect client password.')
                self.redirect('/login')
        else:
            self.show_error_message('Client Does not exist.')
            self.redirect('/login')



"""
Legacy profit functions, shoul be used only for testing new functions.
"""

from operator import attrgetter

from sqlalchemy.orm import joinedload

from gryphon.lib.logger import get_logger
from gryphon.lib.models.order import Order
from gryphon.lib.models.trade import Trade
import gryphon.lib.gryphonfury.positions as positions
import gryphon.lib.gryphonfury.revenue as revenue_lib

logger = get_logger(__name__)


def revenue_fees_profit_in_period(db, start_time, end_time):
    """
    This function is an easy interface to getting revenue, fees, and profit, using
    the identical flow of functions that pentecost presently uses for the trading
    dashboard. This is unused at present but is added here for reference and for
    verifying the fast_(revenue|profit) functions below.
    """

    trades = db\
        .query(Trade)\
        .join(Order)\
        .filter(Order.actor == "Multi")\
        .filter(Trade.time_created >= start_time)\
        .filter(Trade.time_created < end_time)\
        .options(joinedload('order').joinedload('datums'))\
        .all()

    trades.sort(key=attrgetter('time_created'))

    # Get the system position up until the starting point of our graph.
    open_position_offset = positions.fast_position(
        db,
        end_time=start_time,
    )

    position_trades = revenue_lib.open_position_trades(
        open_position_offset,
        db,
        start_time,
    )

    matched_trades, _ = revenue_lib.split_trades(
        position_trades + trades,
    )

    profit, revenue, fees, _ = revenue_lib.profit_data(matched_trades)

    return revenue, fees, profit


def revenue_in_period(db, start_time, end_time):
    """
    Simple function interface to the legacy revenue code. Used for verifying the more
    efficient versions.
    """

    revenue, _, _ = revenue_fees_profit_in_period(db, start_time, end_time)
    return revenue


def profit_in_period(db, start_time, end_time):
    """
    Simple function interface to the legacy profit code. Used for verifying the more
    efficient versions.
    """

    _, _, profit = revenue_fees_profit_in_period(db, start_time, end_time)
    return profit




# -*- coding: utf-8 -*-
from gryphon.data_service.pollers.trades.trades_poller import TradesPoller
from gryphon.lib.money import Money


class BitstampTrades(TradesPoller):
    def __init__(self):
        self.exchange_name = u'BITSTAMP_BTC_USD'
        self.url = 'https://priv-api.bitstamp.net/api/transactions/?time=hour'
        self.poll_time = 10

    def parse_trades(self, resp_obj):
        trades = []

        for raw_trade in resp_obj:
            trade = {
                'exchange': self.exchange_name,
                'price': Money(raw_trade['price'], 'USD'),
                'volume': Money(raw_trade['amount'], 'BTC'),
                'timestamp': int(raw_trade['date']),
                'trade_id': raw_trade['tid'],
            }

            trades.append(trade)

        return trades



# -*- coding: utf-8 -*-
import uuid
from datetime import datetime
import logging

from sqlalchemy import Column, Integer, Unicode, DateTime

from gryphon.dashboards.models.base import Base
from gryphon.dashboards.models.columns.password_column import PasswordColumn

metadata = Base.metadata
logger = logging.getLogger(__name__)


class User(Base):
    __tablename__ = 'user'

    ADMIN_TYPE = 'ADMIN'

    user_id = Column(Integer, primary_key=True)
    unique_id = Column(Unicode(64), nullable=False)
    username = Column(Unicode(64), nullable=False)
    password = Column(PasswordColumn, nullable=False)
    user_type = Column(Unicode(64), nullable=False)
    time_created = Column(DateTime, nullable=False)

    def __init__(self, username, password):
        self.unique_id = uuid.uuid4().hex
        self.time_created = datetime.now()
        self.user_type = User.ADMIN_TYPE
        self.username = username
        # Must be a Password object from models.columns.password_column
        self.password = password



import termcolor as tc
import time

from gryphon.execution.strategies.base import Strategy
from gryphon.execution.lib.exchange_color import exchange_color
from gryphon.lib import session
from gryphon.lib.exchange.exchange_factory import *
from gryphon.lib.money import Money
from gryphon.lib.models.exchange import Exchange as ExchangeData
from gryphon.lib.logger import get_logger

logger = get_logger(__name__)


def get_target(exchange_name):
    db = session.get_a_trading_db_mysql_session()
    try:
        exchange = make_exchange_from_key(exchange_name)
        exchange_data = exchange.exchange_account_db_object(db)
        if not exchange_data:
            exchange_data = ExchangeData(exchange.name)
        target = exchange_data.target['BTC']
        logger.info("%s's target is %s", exchange.friendly_name, target)
    finally:
        db.remove()
    return target

def change_target(exchange_name, target_delta):
    db = get_a_trading_db_mysql_session()

    try:
        exchange = make_exchange_from_key(exchange_name)
        exchange_data = exchange.exchange_account_db_object(db)

        if not exchange_data:
            exchange_data = ExchangeData(exchange.name)

        amount = Money(target_delta, "BTC")
        orig_amount = exchange_data.target[amount.currency]
        exchange_data.target[amount.currency] += amount
        db.add(exchange_data)
        commit_mysql_session(db)

        logger.info("Changed %s target from %s to %s", exchange.friendly_name, orig_amount, exchange_data.target[amount.currency])

    finally:
        db.remove()



import ConfigParser

from gryphon.lib.logger import get_logger

logger = get_logger(__name__)


def get_config_var(filepath, section, key):
    print filepath

    config = ConfigParser.RawConfigParser()
    config.read(filepath)
    section_dict = dict(config.items('live'))

    return section_dict[key]



import pyximport; pyximport.install()

import json
import os
import subprocess

from delorean import epoch
from raven import Client
from sqlalchemy import exc

from gryphon.data_service.consts import *
from gryphon.data_service.queue_consumer import QueueConsumer
from gryphon.lib import session
from gryphon.lib.models.emeraldhavoc.trade import Trade
from gryphon.lib.money import Money

s = Client(dsn=os.environ.get('SENTRY_DSN'))


def trades_consumer_function(message, db):
    subprocess.call(['touch', 'monit/heartbeat/trades_consumer.txt'])
    trade_json = json.loads(message)
    timestamp = epoch(trade_json['timestamp']).datetime
    price_currency = trade_json.get('price_currency', 'USD')
    volume_currency = trade_json.get('volume_currency', 'BTC')

    t = Trade(
        price=Money(trade_json['price'], price_currency),
        volume=Money(trade_json['volume'], volume_currency),
        exchange=unicode(trade_json['exchange']),
        timestamp=timestamp,
        exchange_trade_id=unicode(trade_json['trade_id']),
    )

    db.add(t)

    try:
        session.commit_mysql_session(db)
    except exc.IntegrityError as e:
        # We will get a duplicate entry error from the database if we happen to stop
        # the bot after we write the entry but before we acknowlege it to the queue.
        # This will cause an infinite loop of errors where we keep failing to write the
        # same entry. In this case we can successfully ack the message because we
        # already have it stored.
        if 'Duplicate entry' in str(e):
            return
        else:
            raise e


def main():
    db = session.get_a_gds_db_mysql_session()

    try:
        trades_consumer = QueueConsumer(
            os.environ.get('AMPQ_ADDRESS'),
            trades_consumer_function,
            db,
            EXCHANGE,
            EXCHANGE_TYPE,
            TRADES_BINDING_KEY,
            TRADES_QUEUE,
        )

        trades_consumer.run()
    except KeyboardInterrupt:
        trades_consumer.stop()
    except:
        s.captureException()
    finally:
        db.remove()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
This library provides functions for creating various types of database connections.
"""


def get_mongo_creds():
    import os
    return os.environ['MONGO_DB']


def get_trading_db_mysql_creds():
    import os
    return os.environ['TRADING_DB_CRED']


def get_dashboard_db_mysql_creds():
    import os
    return os.environ['DASHBOARD_DB_CRED']


def get_gds_db_mysql_creds():
    import os
    return os.environ['GDS_DB_CRED']


def get_redis_creds():
    # TODO: this should throw some exception if connection info unavailable.
    import os

    env_var = os.environ.get('REDIS_URL')
    backup_env_var = os.environ.get('REDISTOGO_URL')

    return env_var if env_var else backup_env_var


def get_a_mysql_session(creds):
    import sqlalchemy
    from sqlalchemy.orm import scoped_session, sessionmaker

    engine = sqlalchemy.create_engine(
        creds,
        echo=False,
        pool_size=3,
        pool_recycle=3600,
    )

    session = scoped_session(sessionmaker(bind=engine))

    return session


def get_a_trading_db_mysql_session():
    creds = get_trading_db_mysql_creds()
    return get_a_mysql_session(creds)


def get_a_gds_db_mysql_session():
    creds = get_gds_db_mysql_creds()
    return get_a_mysql_session(creds)


def get_a_dashboard_db_mysql_session():
    creds = get_dashboard_db_mysql_creds()
    return get_a_mysql_session(creds)


def commit_mysql_session(session):
    try:
        session.commit()
    except Exception as e:
        session.rollback()
        raise e


def get_a_mongo_connection():
    from pymongo import MongoClient
    return MongoClient(get_mongo_creds())


def get_a_redis_connection(creds=None):
    """
    This is used for workers to get an adhoc redis connection.
    Call .connection_pool.disconnect() at the end.
    """
    if not creds:
        creds = get_redis_creds()

    import redis

    redis_client = redis.from_url(creds)

    return redis_client


def get_an_rq_connection():
    queue_conn = get_a_redis_connection()
    return queue_conn


def get_a_worker_queue():
    from rq import Queue
    queue_conn = get_an_rq_connection()
    worker_queue = Queue(connection=queue_conn)
    return worker_queue


def get_a_memcache_connection():
    import os
    import pylibmc

    if (os.environ.get('MEMCACHIER_SERVERS')
            and os.environ.get('MEMCACHIER_USERNAME')
            and os.environ.get('MEMCACHIER_PASSWORD')):
        mc = pylibmc.Client(
            servers=os.environ.get('MEMCACHIER_SERVERS').split(','),
            username=os.environ['MEMCACHIER_USERNAME'],
            password=os.environ['MEMCACHIER_PASSWORD'],
            binary=True
        )
    else:
        mc = pylibmc.Client(servers=['localhost:11211'])
    return mc




from collections import OrderedDict

from gryphon.lib.exchange import exchange_factory


def load_exchange_names(db):
    exchanges = exchange_factory.get_all_initialized_exchange_wrappers(db)
    exchange_names = OrderedDict()

    exchanges = sorted(exchanges, key=lambda e: e.friendly_name)

    for exchange in exchanges:
        exchange_names[exchange.name.lower()] = exchange.friendly_name

    return exchange_names



# -*- coding: utf-8 -*-
from gryphon.data_service.pollers.trades.trades_poller import TradesPoller
from gryphon.lib.money import Money


class QuadrigaTrades(TradesPoller):
    def __init__(self):
        self.exchange_name = u'QUADRIGA_BTC_CAD'
        self.url = 'https://api.quadrigacx.com/public/trades?book=btc_cad'
        self.poll_time = 10

    def parse_trades(self, resp_obj):
        trades = []

        for trade in resp_obj:
            # No unique id comes back with the trade so we use the milliseconds
            # timestamp.
            trade = {
                'exchange': self.exchange_name,
                'price': Money(trade['rate'], 'CAD'),
                'volume': Money(trade['amount'], 'BTC'),
                'timestamp': int(trade['datetime']) / 1000.0,
                'trade_id': int(trade['datetime']),
            }

            trades.append(trade)

        return trades



from delorean import epoch

from gryphon.data_service.auditors.orderbook_auditor import OrderbookAuditor


class BitstampOrderbookAuditor(OrderbookAuditor):
    def __init__(self):
        super(BitstampOrderbookAuditor, self).__init__()
        self.exchange_name = 'BITSTAMP_BTC_USD'
        self.orderbook_url = 'https://www.bitstamp.net/api/order_book/'

        self.acceptable_changes_threshold = 40

    def get_timestamp(self, new_orderbook):
        return epoch(float(new_orderbook['timestamp'])).datetime



"""
Tests authenticated and order-placing endpoints.
"""

import pyximport; pyximport.install()
import gryphon.lib; gryphon.lib.prepare()

import logging
import os
import unittest
import sure
import mock
from mock import patch
import time

from gryphon.lib.money import Money
from gryphon.lib.exchange import order_types
from gryphon.lib.exchange.consts import Consts

logger = logging.getLogger(__name__)


class LiveOrdersTest(object):
    def __init__(self):
        self.order1_price_amount = '0.01'
        self.order2_price_amount = '0.02'
        self.sleep_time = 1

    def tearDown(self):
        try:
            self.exchange.cancel_all_open_orders()
            pass
        except:
            logger.error('Could not wind down, there may still be open orders.')

    def test_order_placement_status_and_cancel_methods(self):
        """
        This is a large test that exercises several elements of the wrapper at once,
        because otherwise there would be a lot of duplication between multiple test
        cases. Test here are place_order, get_open_orders, get_order_details,
        get_multi_order_details, cancel_order, and cancel_all_open_orders.
        """

        order_volume = Money('1', self.exchange.volume_currency)
        order1_price = Money(self.order1_price_amount, self.exchange.currency)
        order2_price = Money(self.order2_price_amount, self.exchange.currency)

        # Test the place_order function.

        result = self.exchange.place_order(
            mode=Consts.BID,
            volume=order_volume,
            price=order1_price,
            order_type=order_types.LIMIT_ORDER,
        )

        assert result['success'] is True
        order1_id = result['order_id'] 

        time.sleep(self.sleep_time)

        # Test that place_order had the expected affect through the get_open_orders()
        # function.

        open_orders = self.exchange.get_open_orders()

        assert len(open_orders) == 1
        assert open_orders[0]['id'] == order1_id
        assert open_orders[0]['price'] == order1_price
        assert open_orders[0]['volume_remaining'] == order_volume
      
        time.sleep(self.sleep_time)

        # Test the get_order_details function.

        order_details = self.exchange.get_order_details(order1_id)
        # Due to implementation sometimes get_order_details doesn't have the mode on it.
        assert order_details['type'] in [Consts.BID, None]
        assert order_details['fiat_total'] == Money('0', self.exchange.currency)
        assert len(order_details['trades']) == 0

        time.sleep(self.sleep_time)

        # Place a second order to test other functions.

        result = self.exchange.place_order(
            mode=Consts.BID,
            volume=order_volume,
            price=order2_price,
            order_type=order_types.LIMIT_ORDER,
        )

        assert result['success'] is True
        order2_id = result['order_id'] 

        time.sleep(self.sleep_time)

        # Test that both orders are now in open_orders.

        open_orders = self.exchange.get_open_orders()

        assert len(open_orders) == 2
        assert open_orders[0]['id'] in [order1_id, order2_id]

        time.sleep(self.sleep_time)

        # Test the get_multi_order_details function.

        multi_details = self.exchange.get_multi_order_details([order1_id, order2_id])

        assert len(multi_details) == 2
        assert order1_id in multi_details.keys() and order2_id in multi_details.keys()
        assert 'trades' in multi_details[order2_id]
        assert 'type' in multi_details[order2_id]
        assert 'fiat_total' in multi_details[order2_id]

        time.sleep(self.sleep_time)

        # Test cancelling an order and confirm the expected effect with get_open_orders.
        result = self.exchange.cancel_order(order1_id)

        time.sleep(self.sleep_time)

        open_orders = self.exchange.get_open_orders()

        assert len(open_orders) == 1
        assert open_orders[0]['id'] == order2_id

        # Test cancel_all_open_orders and confirm the expected effect with
        # get_open_orders.
        self.exchange.cancel_all_open_orders()

        time.sleep(self.sleep_time)

        open_orders = self.exchange.get_open_orders()

        assert len(open_orders) == 0




"""
Our first fundamental value version used for the multi algorithm.
"""

from gryphon.lib.exchange.retry import exchange_retry
from gryphon.lib.money import Money


@exchange_retry()
def calculate(algo, fundamental_value_balance_map):
    """
    Using the fundamental value balance map, the core fundamental value is the fiat-
    capital-weighted average of the fundamental values.
    """
    buffer_value = algo.config['volume_currency_buffer_value']

    algo.log(
        'buffer_value:%s USD:%s' % (buffer_value, buffer_value.to('USD')),
        log_level='debug',
    )

    bid_fundamental_value = Money('0', 'USD')
    ask_fundamental_value = Money('0', 'USD')

    bid_participating_exchanges = []
    ask_participating_exchanges = []

    bid_maximum_sum = Money('0', 'USD')
    ask_maximum_sum = Money('0', 'USD')

    # Determine which exchanges should participate in the bid and which in the ask
    # and sum them up appropriately.
    for exchange_name, exchange in fundamental_value_balance_map.iteritems():
        # You may sell if you have enough bitcoin to place an order and don't have
        # too much fiat.
        if (exchange[algo.volume_currency.lower()] > buffer_value and
                exchange['fiat'] + buffer_value.to('USD') < exchange['maximum'] and
                'bid_quote' in exchange and
                exchange['bid_quote']):
            ask_participating_exchanges.append(exchange_name)
            ask_maximum_sum += exchange['maximum']

        # You may buy if you have enough fiat.
        if (exchange['fiat'] > buffer_value.to('USD') and
                'ask_quote' in exchange and
                 exchange['ask_quote']):
            bid_participating_exchanges.append(exchange_name)
            bid_maximum_sum += exchange['maximum']

    for exchange_name in bid_participating_exchanges:
        exchange = fundamental_value_balance_map[exchange_name]
        bid_fundamental_value += (
            exchange['ask_quote'] * (exchange['maximum'] / bid_maximum_sum)
        )

    for exchange_name in ask_participating_exchanges:
        exchange = fundamental_value_balance_map[exchange_name]
        ask_fundamental_value += (
            exchange['bid_quote'] * (exchange['maximum'] / ask_maximum_sum)
        )

    algo.log(
        'Bid Participating Exchanges: %s' % bid_participating_exchanges,
        log_level='debug',
    )

    algo.log(
        'Bid Fundamental Value: %s' % bid_fundamental_value,
        log_level='debug',
    )

    algo.log(
        'Ask Participating Exchanges: %s' % ask_participating_exchanges,
        log_level='debug',
    )

    algo.log(
        'Ask Fundamental value: %s' % ask_fundamental_value,
        log_level='debug',
    )

    if bid_participating_exchanges and ask_participating_exchanges:
        algo.log(
            'Using avg of bid and ask participating exchanges',
            log_level='debug',
        )

        return (bid_fundamental_value + ask_fundamental_value) / 2
    elif bid_participating_exchanges:
        algo.log('Using bid participating exchanges', log_level='debug')

        return bid_fundamental_value
    elif ask_participating_exchanges:
        algo.log('Using ask participating exchanges', log_level='debug')

        return ask_fundamental_value
    else:
        raise Exception('No exchanges to calculate fundamental value')




from gryphon.lib.exchange.base import Exchange
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.money import Money
from gryphon.lib.metrics import quote as quote_lib


class OrderbookSizeException(Exception):
    pass


def calculate(orderbook, depth=Money('10', 'BTC')):
    try:
        bid_quote = quote_lib.price_quote_from_orderbook(
            orderbook,
            Consts.BID,
            depth,
        )

        bid_quote_price = bid_quote['total_price']

        ask_quote = quote_lib.price_quote_from_orderbook(
            orderbook,
            Consts.ASK,
            depth,
        )

        ask_quote_price = ask_quote['total_price']

    except:
        raise OrderbookSizeException

    fundamental_quote = (bid_quote_price + ask_quote_price) / 2
    fundamental_value = fundamental_quote / depth.amount

    return fundamental_value




# import numpy as np
# import time
# 
# sampleData  = open('sampleData.txt','r').read()
# 
# splitData = sampleData.split('\n')
# 
# date,closep,highp,lowp,openp,volume = np.loadtxt(splitData,
#                                                  delimiter=',',
#                                                  unpack=True)
# 
# def movingaverage(values,window):
#     weigths = np.repeat(1.0, window)/window
#     smas = np.convolve(values, weigths, 'valid')
#     return smas # as a numpy array
# 
# def CCI(d,c,h,l,o,v,tf,sma):
#     typPrices = []
#     MDar = []
#     cci = []
# 
#     x = 0
#     while x < len(h):
#         tp = (h[x]+l[x]+c[x])/3
# 
#         typPrices.append(tp)
#         x+=1
# 
#     SMATP = movingaverage(typPrices,sma)
# 
#     #print len(SMATP)
#     typPrices = typPrices[sma-1:]
#     #print len(typPrices)
# 
#     y = tf
# 
#     while y < len(SMATP):
#         considerationTP = typPrices[y-tf:y]
#         considerationSMATP = SMATP[y-tf:y]
# 
#         MDs = 0
#         z = 0
#         while z < len(considerationTP):
#             curMD = abs(considerationTP[z]-considerationSMATP[z])
#             MDs += curMD
#             z+=1
#         MD = MDs/tf
#         MDar.append(MD)
# 
#         y+=1
# 
#     #print len(MDar)
#     typPrices = typPrices[14:]
#     SMATP = SMATP[14:]
#     #print len(typPrices)
#     #print len(SMATP)
# 
#     xx = 0
#     while xx < len(SMATP):
#         ccis = (typPrices[xx]-SMATP[xx]) / (0.015 * MDar[xx])
#         cci.append(ccis)
#         xx+=1
# 
#     print len(cci)
#     print len(date[tf+sma-1:])
#     print cci
#     return date[tf+sma-1:],cci
# 
# CCI(date,closep,highp,lowp,openp,volume,14,20)



"""
Monkeypatch the timeit module so that it returns a function result as well
as the time it took to execute the function. This is used to profile ticks
in production.
"""
import timeit

def _template_func(setup, func):
    """Create a timer function. Used if the "statement" is a callable."""
    def inner(_it, _timer, _func=func):
        setup()
        _t0 = _timer()
        for _i in _it:
            retval = _func()
        _t1 = _timer()
        return _t1 - _t0, retval
    return inner

def monkeypatch_timeit():
    timeit._template_func = _template_func



import numpy as np
import time

sampleData = open('sampleData.txt','r').read()
splitData = sampleData.split('\n')

date,closep,highp,lowp,openp,volume = np.loadtxt(splitData,
                                                 delimiter=',',
                                                 unpack=True)

def aroon(tf):

    AroonUp = []
    AroonDown = []
    AroonDate = []

    x = tf
    #Through some sorcery, we can actually 1 liner this
    while x < len(date):

        #print highp[x]
        #  19, -  since we wanna know how many days SINCE
        # then we create the list with the x-20 to x... then we
        # convert to list since its a numpy array.
        # then we do a .index to find out where the max is at in the list.

        Aroon_Up = tf-(highp[x-(tf-1):x].tolist().index(max(highp[x-(tf-1):x])))
        Aroon_Down = tf-(lowp[x-(tf-1):x].tolist().index(min(lowp[x-(tf-1):x])))

        AroonUp.append(Aroon_Up)
        AroonDown.append(Aroon_Down)
        AroonDate.append(date[x])

        x+=1
    return AroonDate,AroonUp,AroonDown

aroon(20)



"""
You're going to place a limit order in an orderbook at a certain price, but you want to
improve the price in your favour if the orderbook is thin behind the given price. This
function 'slides' the price backwards until it finds a wall of a given strength and places
the order price to be just in front of that wall. Useful for almost any kind of limit-order
trading.

Parameters:
  orderbook, max_slide, current order price/side, jump, ignore_volume.

A couple decisions here:
  - the current max_spread logic does not actually do what it says. Instead, it gives a
    'max distance from the midpoint per order'.
      to do real max_spread we'd have to modify both orders at once
  - does midpoint matter?
      we could just call it 'max slide'

Two known use-cases here:
  - folm
  - multilinear

TODO:
  - the current logic works only on a per-order basis. I believe on some exchanges you 
    could theoretically have a situation where there are 1b 1 satoshi orders at a single
    price level, and these will all appear to be under your ignore volume.

We might make this a 'market making' library if we can think of other things to do here.
"""

from gryphon.lib.exchange.consts import Consts
from gryphon.lib.money import Money


def is_deeper_price(mode, price_a, price_b):
    """
    price_a is deeper than price_b iff were they both on the [mode] side of an
    orderbook, price_a would be further from the center than price_b.
    """
    if mode == Consts.BID:
        return price_a < price_b
    elif mode == Consts.ASK:
        return price_a > price_b


def widen_price(mode, price, amount):
    """Push a price away from the center of the orderbook"""
    if mode == Consts.BID:
        return max(price - amount, Money('0.001', price.currency))
    elif mode == Consts.ASK:
        return price + amount


def narrow_price(mode, price, amount):
    """Push a price away towards the center of the orderbook"""
    if mode == Consts.BID:
        return price + amount
    elif mode == Consts.ASK:
        return price - amount


def slide_order(mode, initial_price, orderbook, ignore_volume=None, jump=None, max_slide=None):
    """
    Sliding an order means to move an order opportunistically deeper into an orderbook
    (to a better price for the trader) given a few conditions.

    We highly recommend always setting max_slide, but if you do not, we have arbitrary
    default boundaries defined as follows:
        bids: 0 + jump 
        asks: (10 * initial price) - jump 
    """

    new_price = None
    orderbook_side = None
    top_opposite_order = None
    volume_currency = orderbook['bids'][0].volume.currency  # Test this better.

    if ignore_volume is None:
        ignore_volume = Money('0.00000001', volume_currency)

    if jump is None:
        jump = Money('0.01', initial_price.currency)

    if max_slide is not None:
        widest_allowed_price = widen_price(mode, initial_price, max_slide)
    else:
        widest_allowed_price = narrow_price(
            mode,
            widen_price(mode, initial_price, 10 * initial_price),
            jump,
        )
    
    if mode == Consts.BID:
        orderbook_side = orderbook['bids']
        top_opposite_order = orderbook['asks'][0]
    elif mode == Consts.ASK:
        orderbook_side = orderbook['asks']
        top_opposite_order = orderbook['bids'][0]

    for order in orderbook_side:
        if (is_deeper_price(mode, order.price, initial_price)
                and order.volume > ignore_volume):

            slid_price = narrow_price(mode, order.price, jump)

            if is_deeper_price(mode, initial_price, slid_price):
                # This catches the case where due to the value of jump, we actually
                # slid to a worse price.
                new_price = initial_price
            elif is_deeper_price(mode, top_opposite_order.price, slid_price):
                # This catches the case where we accidentally crossed the spread.
                new_price = initial_price
            elif is_deeper_price(mode, slid_price, widest_allowed_price):
                # Don't slide past our maximum.
                new_price = widest_allowed_price
            else:
                # Everything checks out.
                new_price = slid_price

            break

    if new_price == None:
        if max_slide != None:
            new_price = widen_price(mode, initial_price, max_slide)
        else:
            new_price = jump

    return new_price



import numpy as np
import time

sampleData = open('sampleData.txt','r').read()
splitData = sampleData.split('\n')

date,closep,highp,lowp,openp,volume = np.loadtxt(splitData, delimiter=',',unpack=True)

def standard_deviation(tf,prices):

    sd = []
    sddate = []
    x = tf
   ######
    while x <= len(prices):
        array2consider = prices[x-tf:x]
        standev = array2consider.std()
        sd.append(standev)
        sddate.append(date[x])
        x+=1
    return sddate,sd

def bollinger_bands(mult,tff):
    bdate = []
    topBand = []
    botBand = []
    midBand = []

    x = tff

    while x < len(date):
        curSMA = movingaverage(closep[x-tff:x],tff)[-1]

        d,curSD = standard_deviation(tff,closep[0:tff])

        curSD = curSD[0]

        #print curSD
        #print curSMA

        TB = curSMA + (curSD*mult)
        BB = curSMA - (curSD*mult)
        D = date[x]

        bdate.append(D)
        topBand.append(TB)
        botBand.append(BB)
        midBand.append(curSMA)

        x+=1

    return bdate,topBand,botBand,midBand

d,tb,bb,mb = bollinger_bands(2,20)



# -*- coding: utf-8 -*-
from datetime import datetime, timedelta
import subprocess

from cdecimal import Decimal
from sqlalchemy import and_, func, select
from twisted.internet import defer
from twisted.internet.defer import inlineCallbacks
from twisted.internet.task import LoopingCall
from twisted.python import log

from gryphon.data_service.auditors.auditor import Auditor
import gryphon.data_service.util as util
from gryphon.lib.models.emeraldhavoc.base import EmeraldHavocBase
from gryphon.lib.models.emeraldhavoc.exchange_volume import ExchangeVolume
from gryphon.lib.models.emeraldhavoc.trade import Trade
from gryphon.lib.slacker import Slacker

metadata = EmeraldHavocBase.metadata
trades = metadata.tables['trade']
exchange_volumes = metadata.tables['exchange_volume']


class TradesAuditor(Auditor):
    def __init__(self, exchanges=[]):
        self.exchanges = exchanges

    def heartbeat(self, response=None):
        subprocess.call(["touch", "monit/heartbeat/trades_auditor_heartbeat.txt"])

    @defer.inlineCallbacks
    def start(self):
        self.redis = yield util.setup_redis()
        self.setup_mysql()
        self.heartbeat_key = 'trades_auditor_heartbeat'
        self.looping_call = LoopingCall(self.audit).start(1800)
        self.accuracy_bottom = Decimal('0.99')
        self.accuracy_top = Decimal('1.01')
        self.slacker = Slacker('#notifications', 'Auditor')

    @inlineCallbacks
    def audit(self):
        successes = []

        for exchange in self.exchanges:
            now = datetime.utcnow()
            twenty_four_hours_ago = now - timedelta(hours=24)
            ten_minutes_ago = now - timedelta(minutes=10)

            trades_sum_result = yield self.engine.execute(
                select([func.sum(trades.c.volume)])
                .where(and_(
                    trades.c.exchange.__eq__(exchange),
                    trades.c.timestamp.between(twenty_four_hours_ago, now),
                ))
            )

            trades_sum_result = yield trades_sum_result.fetchone()
            trades_volume = trades_sum_result[0] or 0

            exchange_volume_result = yield self.engine.execute(
                select([exchange_volumes.c.exchange_volume])
                .where(and_(
                    exchange_volumes.c.exchange.__eq__(exchange),
                    exchange_volumes.c.timestamp.between(ten_minutes_ago, now),
                ))
                .order_by(exchange_volumes.c.timestamp.desc())
            )

            exchange_volume_result = yield exchange_volume_result.fetchone()

            if exchange_volume_result:
                most_recent_exchange_volume = exchange_volume_result[0]

                accuracy = trades_volume / most_recent_exchange_volume
                log.msg('Trade Volume Accuracy on %s: %s' % (exchange, accuracy))

                if accuracy < self.accuracy_bottom or accuracy > self.accuracy_top:
                    successes.append(False)

                    self.slacker.notify(
                        '%s Trades Poller at %s accuracy. Outside %s-%s' % (
                        exchange,
                        accuracy,
                        self.accuracy_bottom,
                        self.accuracy_top,
                    ))
                else:
                    successes.append(True)

        # If all exchanges passed their audit then we heartbeat.
        if all(successes):
            self.heartbeat()



from collections import OrderedDict
import json

from autobahn.twisted.websocket import WebSocketClientFactory
from delorean import Delorean
import treq
from twisted.internet import defer, reactor
from twisted.python import log

import gryphon.data_service.consts as consts
import gryphon.data_service.util as util
from gryphon.data_service.websocket_client import EmeraldWebSocketClientProtocol
from gryphon.data_service.pollers.orderbook.websocket.websocket_orderbook_poller import WebsocketOrderbookPoller
from gryphon.lib.logger import get_logger

logger = get_logger(__name__)


class BitstampOrderbookWebsocket(EmeraldWebSocketClientProtocol, WebsocketOrderbookPoller):
    def __init__(self):
        self.exchange_name = u'BITSTAMP_BTC_USD'

    def connect_to_websocket(self):
        logger.info('Connecting to websocket')

        factory = WebSocketClientFactory(
            'ws://ws.pusherapp.com:80/app/de504dc5763aeef9ff52?protocol=6',
            debug=False,
        )

        # This actually creates a new Instance of BitstampOrderbook.
        factory.protocol = type(self)
        reactor.connectTCP("ws.pusherapp.com", 80, factory)

    def subscribe_to_websocket(self):
        logger.info('Subscribing to websocket')

        data = {
            'channel': 'diff_order_book',
        }

        event = {
            'event': 'pusher:subscribe',
            'data': data,
        }

        self.sendMessage(json.dumps(event))

    # REQUEST POLLER FUNCTIONS

    def get_response(self, response):
        """
        Reimplemented this and handle_response_error from the RequestPoller base class
        because we need to be able to recover from errors that occur in the setup
        process for a websocket poller.  Previously an error in the initial REST call
        would prevent the poller from initializing properly.
        """

        # Parse the Response and check for errors
        d = treq.content(response)

        # We want to add parse_float=Decimal, but it currently breaks json writing
        # in later code paths
        d.addCallback(json.loads)
        d.addCallback(self.parse_response)
        d.addErrback(self.handle_response_error, response)

        return d

    def handle_response_error(self, error_text, response, *args, **kwargs):
        d = treq.text_content(response).addCallback(
            self.log_response,
            u'Error in Response from URL:%s %s' % (self.url, error_text),
            log.err,
        )

        d.addCallback(self.retry_request)

    def retry_request(self, *args):
        log.msg('Retrying initial orderbook call - Looking for Orderbook Timestamp (%s) to be greater than First Change Timestamp (%s)' % (self.orderbook_timestamp, self.first_change_timestamp))

        reactor.callLater(0.5, self.get_request)

    def parse_response(self, resp_obj):
        """
        This function will collect the baseline orderbook from the http endpoint.

        {"timestamp": "1412095328", "bids": [["382.74", "4.85241530"],
        """
        self.orderbook_timestamp = int(resp_obj['timestamp'])

        if (self.orderbook_timestamp > self.first_change_timestamp
                and self.message_count > self.message_count_buffer):
            self.orderbook = {
                'bids': self.parse_orders(resp_obj['bids']),
                'asks': self.parse_orders(resp_obj['asks']),
            }
        else:
            # Get the orderbook again since it was too old.
            self.retry_request()

    # WEBSOCKET CLIENT FUNCTIONS
    @defer.inlineCallbacks
    def onOpen(self):
        logger.info('Connected to websocket')

        self.url = 'https://priv-api.bitstamp.net/api/order_book/'
        self.redis = yield util.setup_redis()

        binding_key = '%s.orderbook.tinker' % self.exchange_name.lower()
        self.producer = yield util.setup_producer(consts.ORDERBOOK_QUEUE, binding_key)

        # Reset everything
        self.orderbook = ''
        self.orderbook_timestamp = None
        self.orderbook_change_backlog = {}
        self.first_change_timestamp = None
        self.message_count = 0
        self.message_count_buffer = 3
        self.last_amqp_push = 0

        yield self.redis.set(self.orderbook_key, None)

        # Start fetching the base orderbook from self.url the request poller will call
        # parse_response with the response.
        self.get_request()

        self.subscribe_to_websocket()

    @defer.inlineCallbacks
    def onMessage(self, payload, isBinary):
        payload = json.loads(payload)

        if payload['event'] == u'data':
            should_continue = yield self.check_should_continue()

            if not should_continue:
                yield self.redis.set(self.orderbook_key, None)
                return

            self.message_count += 1
            orderbook_change = json.loads(payload['data'])

            orderbook_change['bids'] = [
                ['{0:.2f}'.format(float(b[0])), b[1]] for b in orderbook_change['bids']
            ]

            orderbook_change['asks'] = [
                ['{0:.2f}'.format(float(b[0])), b[1]] for b in orderbook_change['asks']
            ]

            current_timestamp = Delorean().epoch

            if not self.first_change_timestamp:
                self.first_change_timestamp = current_timestamp

            if self.orderbook and not self.orderbook_change_backlog:
                # NO Backlog of changes, apply the change.
                self.apply_change_to_orderbook(orderbook_change)

                self.publish_orderbook()

                return
            elif self.orderbook and self.orderbook_change_backlog:
                log.msg('Working down the backlog')

                # Adding current update to backlog.
                self.orderbook_change_backlog[current_timestamp] = orderbook_change

                # Working down the backlog.
                orderbook_backlog_timestamps = sorted(
                    self.orderbook_change_backlog.keys(),
                )

                for ts in orderbook_backlog_timestamps:
                    if ts > self.orderbook_timestamp:
                        log.msg('Applying Change from TS:%s to Orderbook TS:%s' % (
                            ts,
                            self.orderbook_timestamp,
                        ))

                        self.apply_change_to_orderbook(
                            self.orderbook_change_backlog.pop(ts),
                        )
                    else:
                        # This update is too old. get rid of it.
                        self.orderbook_change_backlog.pop(ts)

                self.publish_orderbook()

                return
            else:
                log.msg('Bitstamp Orderbook Not Ready. Orderbook TS:%s must be > %s' % (
                    self.orderbook_timestamp,
                    self.first_change_timestamp,
                ))

                current_timestamp = Delorean().epoch
                self.orderbook_change_backlog[current_timestamp] = orderbook_change

                return
        else:
            log.msg('Payload: %s' % payload)

    # HELPER FUNCTIONS

    def apply_change_to_orderbook(self, change):
        bids_changes = self.parse_orders(change['bids'])
        asks_changes = self.parse_orders(change['asks'])

        # Remove the 0 volumes from the orderbook.
        self.orderbook['bids'].update(bids_changes)

        for k, v in self.orderbook['bids'].iteritems():
            if v == "0":
                self.orderbook['bids'].pop(k)

        # Re-sort the bids.
        self.orderbook['bids'] = OrderedDict(sorted(
            self.orderbook['bids'].iteritems(),
            key=lambda (k, v): float(k),
            reverse=True,
        ))

        self.orderbook['asks'].update(asks_changes)

        for k, v in self.orderbook['asks'].iteritems():
            if v == "0":
                self.orderbook['asks'].pop(k)

        # Re-sort the asks.
        self.orderbook['asks'] = OrderedDict(
            sorted(self.orderbook['asks'].iteritems(), key=lambda (k, v): float(k)),
        )

    def parse_orders(self, orders):
        """Returns a price keyed orders"""
        orders_dict = OrderedDict()

        for order in orders:
            orders_dict[order[0]] = order[1]

        return orders_dict

    def get_orderbook_to_publish(self):
        """Returns a string keyed orderbook from price keyed"""
        price_key_orderbook = self.orderbook

        return {
            'bids': [[k, v, ''] for k, v in price_key_orderbook['bids'].iteritems()],
            'asks': [[k, v, ''] for k, v in price_key_orderbook['asks'].iteritems()],
        }




"""
This package collects code for calculating various features of market data in a single
place. These `metrics` can be used in machine learning, in dashboards, or directly in
strategies. Examples are: the midpoint of an orderbook, or the strength of the bid in an
orderbook at any given depth.

Much of this code use to reside in exchange.base, but it's far outgrown it's place
there.
"""



import json

from delorean import Delorean
from twisted.internet import defer

from gryphon.data_service.pollers.orderbook.orderbook_poller import OrderbookPoller
from gryphon.lib.logger import get_logger

logger = get_logger(__name__)


class WebsocketOrderbookPoller(OrderbookPoller):
    AMQP_PUSH_MAX_FREQUENCY = 1  # Seconds.

    def start(self):
        # Don't do any setup in here, because connect_to_websocket creates a new
        # instance.
        self.connect_to_websocket()

        # We are deliberately avoiding calling super() because we don't want to start
        # a looping call to get_request.

    @defer.inlineCallbacks
    def restart(self):
        yield self.onOpen()

    @defer.inlineCallbacks
    def publish_orderbook(self):
        new_orderbook = self.get_orderbook_to_publish()

        orderbook_has_changed = yield self.check_if_orderbook_has_changed(new_orderbook)

        new_orderbook_with_metadata = {
            'timestamp': Delorean().epoch,
            self.exchange_name: new_orderbook,
        }

        new_orderbook_string = json.dumps(new_orderbook_with_metadata)

        self.heartbeat()

        # We always publish to Redis, even if the orderbook has not changed
        # so that the timestamp gets updated correctly
        logger.debug('Publishing orderbook to Redis')
        yield self.redis.set(self.orderbook_key, new_orderbook_string)

        seconds_since_last_amqp_push = Delorean().epoch - self.last_amqp_push
        if (self.producer and orderbook_has_changed and
                seconds_since_last_amqp_push > self.AMQP_PUSH_MAX_FREQUENCY):
            logger.debug('Publishing orderbook on AMQP')
            self.producer.publish_message(new_orderbook_string)
            self.last_amqp_push = Delorean().epoch

    @defer.inlineCallbacks
    def check_if_orderbook_has_changed(self, new_orderbook):
        orderbook_has_changed = True

        old_orderbook_string = yield self.redis.get(self.orderbook_key)
        if old_orderbook_string and old_orderbook_string != "None":
            old_orderbook = json.loads(old_orderbook_string)

            if old_orderbook[self.exchange_name] == new_orderbook:
                orderbook_has_changed = False

        defer.returnValue(orderbook_has_changed)

    @defer.inlineCallbacks
    def check_should_continue(self):
        should_continue_key = '%s_orderbook_should_continue' % self.exchange_name.lower()
        should_continue = yield self.redis.get(should_continue_key)
        if not should_continue:
            raise KeyError('Expecting %s key in redis.' % should_continue_key)
        should_continue = bool(int(should_continue))

        if not should_continue:
            logger.debug('%s Orderbook Poller respecting Auditor hard failure.' % self.exchange_name)

        defer.returnValue(should_continue)



import pyximport; pyximport.install()
import gryphon.lib; gryphon.lib.prepare()

import os
import unittest
import sure
import mock
from decimal import Decimal
from delorean import Delorean

from gryphon.lib.models.liability import Liability
from gryphon.lib.money import Money
from gryphon.lib.time_parsing import parse


class TestLiability(unittest.TestCase):
    def setUp(self):
        self.basic_liability = Liability(
            Money('100', 'ETH'),
            Liability.FIXED_INTEREST,
            'John',
        )

    def tearDown(self):
        pass

    def test_creation(self):
        l = self.basic_liability

        l.time_started.should.equal(None)
        l.time_repayed.should.equal(None)
        l.amount.should.equal(Money('100', 'ETH'))
        l.entity_name.should.equal('John')

    def test_creation_with_times(self):
        l = Liability(
                Money('100', 'ETH'),
                Liability.FIXED_INTEREST,
                'John',
                time_started=parse('2016-10-1').datetime,
                time_repayed=parse('2016-11-1').datetime,
        )

        l.time_started.should.equal(parse('2016-10-1').datetime)
        l.time_repayed.should.equal(parse('2016-11-1').datetime)
        l.amount.should.equal(Money('100', 'ETH'))
        l.entity_name.should.equal('John')

    def test_start(self):
        l = self.basic_liability

        l.start()

        l.time_started.day.should.equal(Delorean().datetime.day)
        l.time_repayed.should.equal(None)

    def test_end(self):
        l = self.basic_liability

        l.start()
        l.complete()

        l.time_started.day.should.equal(Delorean().datetime.day)
        l.time_repayed.day.should.equal(Delorean().datetime.day)

    def test_creation_with_details(self):
        l = Liability(
                Money('100', 'ETH'),
                Liability.FIXED_INTEREST,
                'John',
                time_started=parse('2016-10-1').datetime,
                details={'interest_rate': 0.05},
        )

        l.details['interest_rate'].should.equal(0.05)

    def test_interest(self):
        l = self.basic_liability

        l.interest_rate = Decimal('0.01')

        l.interest_rate.should.equal(Decimal('0.01'))

    def test_compounding(self):
        l = self.basic_liability

        l.compounding_period = Decimal('12')

        l.compounding_period.should.equal(Decimal('12'))



"""
Several functions around getting the "strength" of an orderbook at a certain depth.

TODO: This library needs to be generalized to multiple currencies.
"""

from gryphon.lib.money import Money
import gryphon.lib.metrics.midpoint as midpoint_lib
import gryphon.lib.metrics.volume_available as available_volume
from gryphon.lib.exchange.consts import Consts


def orderbook_strength_at_slippage(orderbook, order_type, slippage):
    """
    Get the amount of liquidity (bitoin denominated) available within [slippage]
    dollars of the top [order_type].

    Returns a decimal.
    """

    vol = Money('0', 'BTC')

    if order_type == Consts.BID:
        top_bid = orderbook['bids'][0]
        price = top_bid.price - slippage

        vol = available_volume.volume_available_at_price(Consts.ASK, price, orderbook)
    else:
        top_ask = orderbook['asks'][0]
        price = top_ask.price + slippage

        vol = available_volume.volume_available_at_price(Consts.BID, price, orderbook)

    return vol.amount


def orderbook_strength_at_slippages(orderbook, order_type, slippages):
    """
    Get the amount of liquidity (bitcoin denominated) available within [slippage]
    dollars of the top [order_type].

    Slippages should be money objects created by str'ing a float.
    Returns a decimal: decimal dict.
    """

    slippage_lookup = {}

    if order_type == Consts.BID:
        top_bid = orderbook['bids'][0]
        prices = [top_bid.price - slippage for slippage in slippages]

        slippage_lookup = {
            (top_bid.price - slippage).amount: slippage for slippage in slippages
        }

        levels = available_volume.volume_available_at_prices(
            Consts.ASK,
            prices,
            orderbook,
        )
    else:
        top_ask = orderbook['asks'][0]
        prices = [top_ask.price + slippage for slippage in slippages]

        slippage_lookup = {
            (top_ask.price + slippage).amount: slippage for slippage in slippages
        }

        levels = available_volume.volume_available_at_prices(
            Consts.BID,
            prices,
            orderbook,
        )

    levels = {
        slippage_lookup[price].amount: value.amount
        for price, value in levels.items()
    }

    return levels


def orderbook_strength_at_slippage_in_usd(orderbook, order_type, slippage):
    """
    Get the amount of liquidity (bitcoin denominated) available within [slippage]
    dollars of the top [order_type].

    Returns a decimal.
    """
    midpoint = midpoint_lib.get_midpoint_from_orderbook(orderbook, Money('20', 'BTC'))

    vol = Money('0', 'BTC')

    if order_type == Consts.BID:
        top_bid = orderbook['bids'][0]
        price = top_bid.price - slippage

        vol = available_volume.volume_available_at_price(Consts.ASK, price, orderbook)
    else:
        top_ask = orderbook['asks'][0]
        price = top_ask.price + slippage

        vol = available_volume.volume_available_at_price(Consts.BID, price, orderbook)

    return vol.amount * midpoint.amount


def orderbook_strength_at_slippages_in_usd(orderbook, order_type, slippages):
    """
    Same as orderbook_strength_at_slippage but instead of returning the levels as a
    bitcoin-denominated quantity, we return the levels as to the usd value of the
    bitcoins at that level, which we definine as level * 20btc midpoint.

    Returns a decimal: decimal dict.
    """

    midpoint = midpoint_lib.get_midpoint_from_orderbook(orderbook, Money('20', 'BTC'))

    slippage_lookup = {}

    if order_type == Consts.BID:
        top_bid = orderbook['bids'][0]
        prices = [top_bid.price - slippage for slippage in slippages]

        slippage_lookup = {
            (top_bid.price - slippage).amount: slippage for slippage in slippages
        }

        levels = available_volume.volume_available_at_prices(
            Consts.ASK,
            prices,
            orderbook,
        )
    else:
        top_ask = orderbook['asks'][0]
        prices = [top_ask.price + slippage for slippage in slippages]

        slippage_lookup = {
            (top_ask.price + slippage).amount: slippage for slippage in slippages
        }

        levels = available_volume.volume_available_at_prices(
            Consts.BID,
            prices,
            orderbook,
        )

    levels = {
        slippage_lookup[price].amount: value.amount * midpoint.amount
        for price, value in levels.items()
    }

    return levels




from admin_base import AdminBaseHandler
import tornado.web

from mixins.start_and_end_time import StartAndEndTimeMixin
from tinkerpy.exchange.exchange_factory import all_exchanges
import util.tick_times as tick_times


class TickTimesHandler(AdminBaseHandler, StartAndEndTimeMixin):
    @tornado.web.authenticated
    def get(self):
        start_time, end_time = self.get_start_time_and_end_time()

        all_tick_time_series = self.get_all_ticktime_series(start_time, end_time)

        self.render_template(
            'tick_times.html',
            args={
                'all_tick_time_series': all_tick_time_series,
            },
        )

    def get_all_ticktime_series(self, start_time, end_time):
        exchange_list = all_exchanges()

        all_tick_time_series = {}

        for exchange in exchange_list:
            tick_time_series = tick_times.get_ticktime_series_for_exchange(
                self.trading_db,
                exchange.name,
                start_time,
                end_time,
            )

            all_tick_time_series[exchange.name] = tick_time_series

        return all_tick_time_series



import tornado.web

from gryphon.dashboards.handlers.admin_base import AdminBaseHandler


class NotFoundHandler(AdminBaseHandler):
    @tornado.web.authenticated
    def get(self):
        self.render_template('404.html')



# -*- coding: utf-8 -*-
from collections import defaultdict
from datetime import datetime, date, timedelta
import json
import os
import uuid

from decimal import *
from sqlalchemy import ForeignKey, Column, Integer, Unicode, DateTime, UnicodeText, Numeric
from sqlalchemy.orm import relationship, backref
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.ext.hybrid import hybrid_property
from sqlalchemy import func
from sqlalchemy.sql import expression
from sqlalchemy import and_

from gryphon.lib import gryphon_json_serialize
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.exchange_factory import make_exchange_from_key
from gryphon.lib.models.base import Base
from gryphon.lib.money import Money

metadata = Base.metadata


class Trade(Base):
    __tablename__ = 'trade'
    
    #Trade Types
    BID = Consts.BID
    ASK = Consts.ASK
    
    trade_id = Column(Integer, primary_key=True)
    trade_type = Column(Unicode(64))
    unique_id = Column(Unicode(64), nullable=False)
    exchange_trade_id = Column(Unicode(64))
    time_created = Column(DateTime, nullable=False)
    
    _fee = Column('fee', Numeric(precision=24, scale=14))
    _fee_currency = Column('fee_currency', Unicode(3))
    _price = Column('price', Numeric(precision=24, scale=14))
    _price_currency = Column('price_currency', Unicode(3))
    _volume = Column('volume', Numeric(precision=24, scale=14))
    _volume_currency = Column('volume_currency', Unicode(3))

    meta_data = Column('meta_data', UnicodeText(length=2**31))

    # Some Trades have BTC fees, which reduce our total BTC assets. Every once in a while
    # we create a "buyback" transaction where we buy back those fees outside our trading system.
    # We then mark the fee as "bought back" by setting the trade's fee_buyback_transaction
    fee_buyback_transaction_id = Column(Integer, ForeignKey('transaction.transaction_id'))
    fee_buyback_transaction = relationship("Transaction",  backref='fee_buyback_trades')

    order_id = Column(Integer, ForeignKey('order.order_id'))
    
    def __init__(self, trade_type, price, fee,  volume, exchange_trade_id, order, meta_data={}):
        self.unique_id = unicode(uuid.uuid4().hex)
        self.time_created = datetime.utcnow()
        self.trade_type = trade_type
        self.price = price
        self.fee = fee
        self.volume = volume
        self.exchange_trade_id = exchange_trade_id
        self.order = order
        self.meta_data = json.dumps(meta_data)
        
    def __unicode__(self):
        return u'[TRADE:%s, Order:%s] Price:%s, Volume:%s BTC, Exchange:%s' % (
            self.trade_type, self.order_id, self.price, self.volume, self.order.exchange.name)
        
    def __repr__(self):
        return self.to_json()
    
    def to_json(self):
        return json.dumps({
            'trade_id':self.trade_id,
            'trade_type':self.trade_type,
            'time_created':unicode(self.time_created),
            'unique_id':self.unique_id,
            'exchange_trade_id':self.exchange_trade_id,
            'order_id':self.order_id,
            'fee':self.fee,
            'price':self.price,
            'volume':self.volume
        }, ensure_ascii=False)
    
    @property
    def volume(self):
        return Money(self._volume, self._volume_currency)

    @volume.setter
    def volume(self, value):
        self._volume = value.amount
        self._volume_currency = value.currency

    @property
    def price(self):
        return Money(self._price, self._price_currency)

    @price.setter
    def price(self, value):
        self._price = value.amount
        self._price_currency = value.currency

    @hybrid_property
    def price_in_usd(self):
        """
        Regular property you can call on a loaded trade in python. Returns a USD Money object
        """
        exchange_rate_to_usd = self.exchange_rate
        return Money(self._price * exchange_rate_to_usd, 'USD')

    @price_in_usd.expression
    def price_in_usd(cls):
        """
        SQL Expression which lets us calculate and use USD prices in SQL.

        Must have Order joined in queries where it is used.
        Ex: db.query(func.sum(Trade.price_in_usd)).join(Order).scalar()
        This gives decimal results, since we don't have our Money objects in SQL
        """
        from gryphon.lib.models.order import Order

        exchange_rate_to_usd = Order.exchange_rate
        return cls._price * exchange_rate_to_usd

    @property
    def fee(self):
        return Money(self._fee, self._fee_currency)

    @fee.setter
    def fee(self, value):
        self._fee = value.amount
        self._fee_currency = value.currency

    @hybrid_property
    def fee_in_usd(self):
        """
        Regular property you can call on a loaded trade in python. Returns a USD Money object
        """
        if self._fee_currency == 'BTC':
            exchange_rate_to_usd = self.fundamental_value.amount * self.exchange_rate
        else:
            exchange_rate_to_usd = self.exchange_rate

        return Money(self._fee * exchange_rate_to_usd, 'USD')

    @fee_in_usd.expression
    def fee_in_usd(cls):
        """
        SQL Expression which lets us calculate and use USD fees in SQL (usually for summing).

        Must have Order joined in queries where it is used.
        Ex: db.query(func.sum(Trade.fee_in_usd)).join(Order).scalar()
        This gives decimal results, since we don't have our Money objects in SQL
        """
        from gryphon.lib.models.order import Order

        exchange_rate_to_usd = expression.case(
            [(cls._fee_currency == 'BTC', Order._fundamental_value * Order.exchange_rate)],
            else_=Order.exchange_rate,
        )

        return cls._fee * exchange_rate_to_usd

    @hybrid_property
    def has_outstanding_btc_fee(self):
        return self._fee_currency == 'BTC' and self.fee_buyback_transaction_id == None

    @has_outstanding_btc_fee.expression
    def has_outstanding_btc_fee(cls):
        return and_(cls._fee_currency == 'BTC', cls.fee_buyback_transaction_id == None)

    # Included for backwards compatibility. New code should call calc_position()
    @property
    def position(self):
        return self.calc_position()

    def calc_position(self, include_fees=True):
        from gryphon.lib.models.exchange import Position
        position = Position()

        if self.trade_type == self.BID:
            position -= self.price
            position += self.volume
        else: #ASK
            position += self.price
            position -= self.volume

        if include_fees:
            position -= self.fee

        return position

    # kinda janky, this is used when we are copying trade objects and need to keep their
    # exchange_rate around without references to their parent order.
    # used in gryphon.lib.gryphonfury.profit:copy_trade()
    @property
    def exchange_rate(self):
        if self.order:
            return self.order.exchange_rate
        elif self._exchange_rate:
            return self._exchange_rate
        return None

    @property
    def fundamental_value(self):
        if self.order:
            return self.order.fundamental_value
        elif self._fundamental_value:
            return self._fundamental_value
        return None

    @property
    def exchange_name(self):
        if self.order:
            return self.order._exchange_name
        elif self._exchange_name:
            return self._exchange_name
        return None

    def convert_to_currency(self, amount, currency):
        if amount.currency == currency:
            return amount
        elif currency == "USD":
            return amount.to("USD", exchange_rate_to_usd=self.exchange_rate)
        else:
            raise ValueError(
                "Cannot to convert trade price into %s (Only USD supported)" % currency,
            )

    def price_in_currency(self, currency):
        return self.convert_to_currency(self.price, currency)

    def fee_in_currency(self, currency):
        if self.fee.currency == "BTC":
            fundamental_value = self.fundamental_value
            # converts BTC fee into currency of fundamental_value
            fee = fundamental_value * self.fee.amount
        else:
            fee = self.fee
        return self.convert_to_currency(fee, currency)



# -*- coding: utf-8 -*-
from gryphon.data_service.pollers.trades.trades_poller import TradesPoller
from gryphon.lib.money import Money
from gryphon.lib.time_parsing import parse


class CoinbaseCadTrades(TradesPoller):
    def __init__(self):
        self.exchange_name = u'COINBASE_BTC_CAD'
        self.url = 'https://api.exchange.coinbase.com/products/BTC-CAD/trades'
        self.poll_time = 10

    def parse_trades(self, resp_obj):
        trades = []

        for trade in resp_obj:
            trade = {
                'exchange': self.exchange_name,
                'price': Money(trade['price'], 'CAD'),
                'volume': Money(trade['size'], 'BTC'),
                'timestamp': parse(trade['time']).epoch,
                'trade_id': trade['trade_id'],
            }

            trades.append(trade)

        return trades



# -*- coding: utf-8 -*-
import os
import requests
import random
from delorean import epoch, Delorean
from collections import OrderedDict
from datetime import datetime, timedelta
from gryphon.lib.money import Money
from gryphon.lib.cache import cache_me

from gryphon.lib.logger import get_logger
logger = get_logger(__name__)

class BitcoinWisdom(object):
    def __init__(self, exchange='bitstamp'):
        self.base_url = 'https://s5.bitcoinwisdom.com/%s'
        self.exchange = exchange.lower()
        self.symbol = 'btcusd'
        if self.exchange == 'cavirtex':
            self.symbol = 'btccad'
        elif self.exchange == 'vaultofsatoshi':
            self.exchange = 'vos'
            self.symbol = 'btccad'
        elif self.exchange == 'kraken':
            self.symbol = 'btceur'
        
        self.steps ={
                '1m': 60,'3m': 180,'5m': 300,'15m': 900,'30m':1800,
                '1h':3600,'2h':7200,'4h':14400,'6h':21600,'12h':43200,
                '1d':86400,'3d':259200,'1w':604800
            }

    # currently caching for a day, maybe longer in the future
    @cache_me(time=86400, ignore_self=True)
    def req_with_cache(self, url):        
        return self.req(url)

    def req(self, url):
        url = url + ('&nonce=%s' % 5408178796808)
        try:
            return requests.get(url).json()
        except ValueError:
            return None

    def create_url(self, step):
        step_seconds = self.steps[step]
        url = 'period?step=%s&symbol=%s%s' % (step_seconds, self.exchange, self.symbol)
        url = self.base_url % url
        return url
    
    def determine_step(self, start_date, end_date):
        if (end_date - start_date) >= timedelta(days=1):
            step = '1d'
        elif (end_date - start_date) >= timedelta(hours=1):
            step='1h'
        else:
            step = '1m'
        return step
    
   
    def period(self, start_date, end_date, step='1d', sid=''):
        if self.should_cache_this_result(start_date, end_date):
            points = self.req_with_cache(self.create_url(step))
        else:
            points = self.req(self.create_url(step))

        if not points:
            return OrderedDict()
        point_hash = OrderedDict()
        for p in points:
            point_hash[epoch(p[0]).naive] = Money(str(p[7]), 'BTC')
        return point_hash
    

    def volume_in_period(self, start_date, end_date):
        start_date = Delorean(start_date, 'UTC').datetime
        end_date = Delorean(end_date, 'UTC').datetime
        total_volume = Money(0, 'BTC')

        periods = self.period(
            start_date,
            end_date,
            step=self.determine_step(start_date, end_date),
        )

        for k,v in periods.iteritems():
            t=Delorean(k, 'UTC').datetime
            vol=v
            if t >= start_date and t < end_date:
                total_volume += vol

        return total_volume


    def should_cache_this_result(self, start_date, end_date):
        five_hours_ago = Delorean(datetime.now() - timedelta(hours=5), 'UTC').datetime

        return end_date < five_hours_ago





import pyximport; pyximport.install()

from gryphon.lib.exchange.coinbase_btc_usd import CoinbaseBTCUSDExchange
from gryphon.tests.environment.exchange_wrappers.live_orders import LiveOrdersTest


class TestCoinbaseBTCUSDLiveOrders(LiveOrdersTest):
    def setUp(self):
        self.exchange = CoinbaseBTCUSDExchange()



import bcrypt
import re
import logging
import sqlalchemy.types as types

bcrypt_matcher = re.compile(r'\A(?P<salt>\$2a?\$\d{2}\$[./0-9a-zA-Z]{22})(?P<hashed>[./0-9a-zA-Z]{31})\Z').match

class Password(object):
    DIFFICULTY = 12

    def __init__(self, value=None, **kwargs):
        self._hashed = kwargs.get('hashed', None)
        self._plain = kwargs.get('plain', None)
        self._salt = kwargs.get('salt', None)

        if self._plain:
            self._plain = self._plain.encode('utf-8')

        if self._salt:
            self._salt = self._salt.encode('utf-8')

        if value:
            match = bcrypt_matcher(value)

            if match:
                self._salt = match.group('salt')
                self._hashed = match.group('hashed')
            else:
                self._plain = value

    def __eq__(self, other):
        if isinstance(other, Password):
            return self.hashed == other.hashed
        else:
            return self.__eq__(Password(plain=other, salt=self.salt))

    @property
    def hashed(self):
        if not self._hashed:
            self._hashed = bcrypt.hashpw(self.plain, self.salt)
        return self._hashed

    @property
    def salt(self):
        if not self._salt:
            if self._hashed:
                self._salt = self._hashed[:30]
            else:
                self._salt = bcrypt.gensalt(log_rounds=self.DIFFICULTY)
        return self._salt

    @property
    def plain(self):
        return self._plain

    def __str__(self):
        return self.hashed

class PasswordColumn(types.TypeDecorator):
    impl = types.String(100)

    def process_bind_param(self, value, dialect):
        return value.hashed

    def process_result_value(self, value, dialect):
        return Password(hashed=value)




import csv
from datetime import timedelta
import gzip
import StringIO

from delorean import epoch
import requests
from sqlalchemy.sql import func

from gryphon.lib import session
from gryphon.lib.bitcoinwisdom import BitcoinWisdom
from gryphon.lib.exchange_volume_lib import get_single_exchange_volume_on_day_from_db_tickers
from gryphon.lib.models.emeraldhavoc.trade import Trade
from gryphon.lib.money import Money
from gryphon.lib.time_parsing import parse


historical_data_url = 'http://api.bitcoincharts.com/v1/csv/%s%s.csv.gz'

historically_available_exchanges = [
    ['BITSTAMP', 'bitstamp', 'USD', 'BTC'],
    ['BITFINEX', 'bitfinex', 'USD', 'BTC'],
    ['COINBASE', 'coinbase', 'USD', 'BTC'],
    ['COINBASE_CAD', 'coinbase', 'CAD', 'BTC'],
    ['ITBIT', 'itbit', 'USD', 'BTC'],
    ['KRAKEN', 'kraken', 'EUR', 'BTC'],
]

all_exchanges = [
    'BITSTAMP',
    'BITFINEX',
    'KRAKEN',
    'COINBASE',
    'COINBASE_CAD',
    'ITBIT',
    'CAVIRTEX',
    'QUADRIGA',
    #'COINSETTER',
]

bw_exchanges = [
    'BITSTAMP',
    'BITFINEX',
    'KRAKEN',
]

test_start_date = parse('2015-12-1').datetime
test_end_date = parse('2015-12-15').datetime

def backfill_trades(exchanges):
    for e in exchanges:
        trades = get_trades_to_backfill(e[0], e[1], e[2], e[3])
        write_trades_to_db(e[0], trades)


def get_trades_to_backfill(our_exchange_id, exchange, price_currency, volume_currency):
    our_trades = get_our_recorded_trades(our_exchange_id.upper())
    oldest_trade_timestamp = our_trades[0][0]
    historical_trades = get_historical_trades(exchange.lower(), price_currency, volume_currency)
    trades_to_add = [t for t in historical_trades if t[0] < oldest_trade_timestamp]

    # sort the trades to add in adding the most recent one first.
    sorted_trades = sorted(trades_to_add, key=lambda k: k[0], reverse=True)

    return sorted_trades


def write_trades_to_db(our_exchange_id, trades_list):
    db = session.get_a_gds_db_mysql_session()
    try:
        for i in range(len(trades_list)):
            trade = Trade(
                trades_list[i][1],
                trades_list[i][2],
                our_exchange_id,
                trades_list[i][0],
                None,
                source='BITCOINCHARTS'
            )
            db.add(trade)
            if i % 30 == 0:
                session.commit_mysql_session(db)

        session.commit_mysql_session(db)

    finally:
        db.remove()


def get_historical_trades(exchange, price_currency, volume_currency='BTC'):
    r = requests.get(historical_data_url % (exchange, price_currency))
    compressedFile = StringIO.StringIO()
    compressedFile.write(r.content)
    compressedFile.seek(0)
    decompressedFile = gzip.GzipFile(fileobj=compressedFile, mode='rb')

    try:
        reader = csv.reader(decompressedFile)
        trades = []
        for row in reader:
            timestamp = epoch(int(row[0])).datetime.replace(tzinfo=None)
            price = Money(row[1], price_currency)
            volume = Money(row[2], volume_currency)
            trades.append([timestamp, price, volume])

        return trades

    finally:
        decompressedFile.close()


def get_our_recorded_trades(exchange):
    db = session.get_a_gds_db_mysql_session()
    try:
        trades_for_ex = db.query(Trade).\
            filter_by(exchange=exchange.upper()).\
            order_by(Trade.exchange_trade_id).all()
        norm_trades = [[t.timestamp.replace(tzinfo=None), t.price, t.volume] for t in trades_for_ex]

        return norm_trades

    finally:
        db.remove()


def get_our_recorded_exchange_trade_volume_for_period(exchange, start_date, end_date):
    db = session.get_a_gds_db_mysql_session()
    try:
        exchange_volume = db.query(func.sum(Trade._volume).label('exchange_volume'))\
            .filter_by(exchange=exchange.upper())\
            .filter(Trade.timestamp >= start_date)\
            .filter(Trade.timestamp < end_date).first()
        return Money(exchange_volume[0], 'BTC')

    finally:
        db.remove()


def get_our_recorded_ticker_volume_for_period(exchange, start_date, end_date):
    db = session.get_a_gds_db_mysql_session()
    try:

        d = start_date
        volume = Money(0, 'BTC')
        while d < end_date:
            volume += get_single_exchange_volume_on_day_from_db_tickers(
                db,
                exchange,
                start_date,
            )
            d += timedelta(days=1)

        return volume

    finally:
        db.remove()


# auditing functions

def audit_ticker_volume_individual_days(exchange_list, start_date=test_start_date, end_date=test_end_date):
    d = start_date
    while d < test_end_date:
        print '\n\nAuditing: %s' % d
        day_after = d + timedelta(days=1)
        audit_all_ticker_volume(exchange_list, d, day_after)
        d = day_after

def audit_all_ticker_volume(exchange_list, start_date=test_start_date, end_date=test_end_date):
    for exchange in exchange_list:
        audit_ticker_volume(exchange, start_date, end_date)


def audit_all_bw_volume(exchange_list):
    start_date = parse('2015-12-01').datetime
    end_date = parse('2015-12-15').datetime
    for exchange in exchange_list:
        audit_bw_volume(exchange, start_date, end_date)


def audit_ticker_volume(exchange, start_date, end_date):
    ticker_volume = get_our_recorded_ticker_volume_for_period(
        exchange,
        start_date,
        end_date,
    )

    our_volume = get_our_recorded_exchange_trade_volume_for_period(
        exchange,
        start_date,
        end_date,
    )

    print '%s  Our Volume:%s  Ticker Volume:%s, Accuracy: %s' % (
        exchange,
        our_volume,
        ticker_volume,
        our_volume / ticker_volume,
    )


def audit_bw_volume(exchange, start_date, end_date):
    bw_exchange = BitcoinWisdom(exchange=exchange)
    bw_volume = bw_exchange.volume_in_period(start_date, end_date)

    our_volume = get_our_recorded_exchange_trade_volume_for_period(exchange, start_date, end_date)
    print '%s  Our Volume:%s  BW Volume:%s, Accuracy: %s' % (exchange, our_volume, bw_volume, our_volume / bw_volume)


def compare_all_exchanges():
    for ex in historically_available_exchanges:
        compare_ours_to_history(ex[0], ex[1], ex[2], ex[3])


def compare_ours_to_history(our_exchange_id, exchange, price_currency, volume_currency='BTC'):
    hist_trades = get_historical_trades(
        exchange.lower(),
        price_currency, volume_currency,
    )
    our_trades = get_our_recorded_trades(our_exchange_id.upper())

    start = parse('2015-11-27 0:0:0').datetime.replace(tzinfo=None)
    end = parse('2015-11-27 11:59:59').datetime.replace(tzinfo=None)

    print our_exchange_id.upper()
    hist_in_range = [t for t in hist_trades if t[0] >= start and t[0] <= end]
    ours_in_range = [t for t in our_trades if t[0] >= start and t[0] <= end]

    for t in hist_in_range:
        if t not in ours_in_range:
            print 'Hist Trade not in ours: %s' % t

    for t in ours_in_range:
        if t not in hist_in_range:
            print'Our trade not in history: %s' % t
    print'\n\n\n\n\n'



from __future__ import with_statement
import pyximport; pyximport.install()
from alembic import context
from sqlalchemy import engine_from_config, pool, create_engine
from logging.config import fileConfig
import logging

logger = logging.getLogger(__name__)


# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
fileConfig(config.config_file_name)

# add your model's MetaData object here
# for 'autogenerate' support
# from myapp import mymodel
# target_metadata = mymodel.Base.metadata

# for some reason need both of these for imports to work
import os, sys
lib_path = os.path.abspath('.')
sys.path.append(lib_path)

from gryphon.lib.models.base import Base
from gryphon.lib.models.flag import Flag
from gryphon.lib.models.order import Order
from gryphon.lib.models.trade import Trade
from gryphon.lib.models.exchange import Exchange
from gryphon.lib.models.event import Event
from gryphon.lib.models.transaction import Transaction
from gryphon.lib.models.datum import Datum
from gryphon.lib.models.liability import Liability

from gryphon.execution.models.backtesting.result_trade import ResultTrade
from gryphon.execution.models.backtesting.result import Result

target_metadata = Base.metadata


# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.

def run_migrations_offline():
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    url = os.environ.get('TRADING_DB_CRED')
    context.configure(url=url)

    with context.begin_transaction():
        context.run_migrations()

def run_migrations_online():
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """
    
    engine = create_engine(os.environ.get('TRADING_DB_CRED'))
        
    # engine = engine_from_config(
    #             config.get_section(config.config_ini_section),
    #             prefix='sqlalchemy.',
    #             poolclass=pool.NullPool)

    connection = engine.connect()
    context.configure(
                connection=connection,
                target_metadata=target_metadata
                )

    try:
        with context.begin_transaction():
            context.run_migrations()
    finally:
        connection.close()

if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()




EXCHANGE = 'trades_exchange'
EXCHANGE_TYPE = 'topic'
TRADES_QUEUE = 'trades_durable'
TRADES_BINDING_KEY = '*.trades.tinker'
ORDERBOOK_QUEUE = 'orderbook_durable'
ORDERBOOK_BINDING_KEY = '*.orderbook.tinker'
EXCHANGE_VOLUME_QUEUE = 'exchange_volume_durable'
EXCHANGE_VOLUME_BINDING_KEY = '*.exchange_volume.tinker'

TRADES_QUEUE_OLD = 'trades'
ORDERBOOK_QUEUE_OLD = 'orderbook'
EXCHANGE_VOLUME_QUEUE_OLD = 'exchange_volume'



import pyximport; pyximport.install()

import os
import mock
from delorean import Delorean
from cdecimal import Decimal

from gryphon.lib.exchange.consts import Consts
from gryphon.lib.money import Money

from algos.naive import Naive
from algos.harness import Harness
from gryphon.lib.models.order import Order
from gryphon.lib.models.exchange import Position, Balance
from algos.base import Strategy
from brain import Brain
from nose.plugins.skip import SkipTest

import logging
logger = logging.getLogger(__name__)

class TestFundamentalValue():

    def setUp(self):
        self.db = mock.Mock()

        exchange = mock.Mock()
        exchange.name = "TEST"
        exchange.friendly_name = "TestExchange"

        other_exchange = mock.Mock()
        other_exchange.name = "COINBASE"
        other_exchange.friendly_name = "OtherExchange"

        params = {
            'max_fiat_balance': "USD 2000"
        }
        self.algo = Strategy(exchange, self.db, params=params)

        self.algo.fundamental_exchanges = [exchange, other_exchange]

        self.harness = Harness(self.algo, self.db)

        self.fundamental_value_map = {
            'TEST':{
                'fiat':Money(1000, 'USD'),
                'btc':Money(10, 'BTC'),
                'fundamental_value': Money(100, 'USD'),
                'bid_quote':Money(99, 'USD'),
                'ask_quote':Money(101, 'USD'),
                'maximum':Money(2000, 'USD')
            },
            'COINBASE':{
                'fiat':Money(1000, 'USD'),
                'btc':Money(10, 'BTC'),
                'fundamental_value': Money(120, 'USD'),
                'bid_quote':Money(119, 'USD'),
                'ask_quote':Money(121, 'USD'),
                'maximum':Money(2000, 'USD')
            }
        }

    def tearDown(self):
        pass

    def test_cfv_equal_capital_equal_maximum(self):
        core_fundamental_value = self.harness.calculate_core_fundamental_value(self.fundamental_value_map)
        core_fundamental_value.should.equal(Money("110", "USD"))

    def test_cfv_unequal_capital_equal_maximum(self):
        # the unequal capital should have no effect
        self.fundamental_value_map['TEST'].update({
            'fiat':Money(750, 'USD'),
        })
        self.fundamental_value_map['COINBASE'].update({
            'fiat':Money(1250, 'USD'),
        })

        core_fundamental_value = self.harness.calculate_core_fundamental_value(self.fundamental_value_map)
        core_fundamental_value.should.equal(Money("110", "USD"))

    def test_cfv_equal_capital_unequal_maximum(self):
        # should take 110 at weight 1/10 and 120 at weight 9/10 = 119
        self.fundamental_value_map['TEST'].update({
            'maximum':Money(1000, 'USD')
        })
        self.fundamental_value_map['COINBASE'].update({
            'maximum':Money(9000, 'USD')
        })

        core_fundamental_value = self.harness.calculate_core_fundamental_value(self.fundamental_value_map)
        core_fundamental_value.should.equal(Money("119", "USD"))

    def test_cfv_low_fiat(self):
        # should ignore TEST's bid_quote since we have no fiat to buy with
        # bid_quote = 119
        # ask_quote = avg(101, 121) = 111
        # cfv = avg(119, 111) = 115
        self.fundamental_value_map['TEST'].update({
            'fiat':Money(0, 'USD'),
        })

        core_fundamental_value = self.harness.calculate_core_fundamental_value(self.fundamental_value_map)
        core_fundamental_value.should.equal(Money("115", "USD"))

    def test_cfv_high_fiat(self):
        # should ignore COINBASE's ask_quote since we have max fiat
        # bid_quote = avg(99, 119) = 109
        # ask_quote = 101
        # cfv = avg(109, 101) = 105
        self.fundamental_value_map['COINBASE'].update({
            'fiat':Money(10000, 'USD'),
        })

        core_fundamental_value = self.harness.calculate_core_fundamental_value(self.fundamental_value_map)
        core_fundamental_value.should.equal(Money("105", "USD"))

    def test_cfv_low_btc(self):
        # same math as above
        self.fundamental_value_map['COINBASE'].update({
            'btc':Money(1, 'BTC'),
        })

        core_fundamental_value = self.harness.calculate_core_fundamental_value(self.fundamental_value_map)
        core_fundamental_value.should.equal(Money("105", "USD"))

    # V2
    def test_cfv_v2_equal_capital_equal_maximum(self):
        core_fundamental_value, __ = self.harness.calculate_core_fundamental_value_v2(self.fundamental_value_map)
        core_fundamental_value.should.equal(Money("110", "USD"))

    def test_cfv_v2_low_fiat(self):
        # should ignore TEST entirely since we have no fiat to buy with
        # COINBASE fv = 120
        self.fundamental_value_map['TEST'].update({
            'fiat':Money(0, 'USD'),
        })

        core_fundamental_value, __ = self.harness.calculate_core_fundamental_value_v2(self.fundamental_value_map)
        core_fundamental_value.should.equal(Money("120", "USD"))

    def test_cfv_v2_high_fiat(self):
        # should ignore TEST entirely since we have max fiat
        # COINBASE fv = 120
        self.fundamental_value_map['TEST'].update({
            'fiat':Money(10000, 'USD'),
        })

        core_fundamental_value, __ = self.harness.calculate_core_fundamental_value_v2(self.fundamental_value_map)
        core_fundamental_value.should.equal(Money("120", "USD"))

    def test_cfv_v2_low_btc(self):
        # should ignore TEST entirely since we have low btc
        # COINBASE fv = 120
        self.fundamental_value_map['TEST'].update({
            'btc':Money(1, 'BTC'),
        })

        core_fundamental_value, __ = self.harness.calculate_core_fundamental_value_v2(self.fundamental_value_map)
        core_fundamental_value.should.equal(Money("120", "USD"))



# -*- coding: utf-8 -*-
import base64
from collections import OrderedDict
import decimal
import hashlib
import hmac
import json
import time

from cdecimal import *

from base import *
from exceptions import *
from gryphon.lib.exchange.gemini_btc_usd import GeminiBTCUSDExchange
from gryphon.lib.models.exchange import Balance
from gryphon.lib.money import Money

from gryphon.lib.logger import get_logger
logger = get_logger(__name__)


class GeminiETHUSDExchange(GeminiBTCUSDExchange):
    def __init__(self, session=None, configuration=None):
        super(GeminiETHUSDExchange, self).__init__(session)

        self.name = u'GEMINI_ETH_USD'
        self.friendly_name = u'Gemini ETH-USD'
        self.currency = 'USD'
        self.volume_currency = 'ETH'
        self.volume_decimal_precision = 6
        self.gemini_pair_symbol = 'ethusd'

        self.fiat_balance_tolerance = Money('0.0001', 'USD')
        self.volume_balance_tolerance = Money('0.00000001', 'ETH')
        self.min_order_size = Money('0.00001', 'ETH')

        if configuration:
            self.configure(configuration)
        



# -*- coding: utf-8 -*-
from gryphon.data_service.pollers.trades.kraken_trades_poller import KrakenTrades


class KrakenCADTrades(KrakenTrades):
    def __init__(self):
        self.exchange_name = u'KRAKEN_BTC_CAD'
        self.currency = 'CAD'
        self.poll_time = 10




from gryphon.data_service.pollers.orderbook.orderbook_poller import OrderbookPoller


class BitstampBTCEUROrderbook(OrderbookPoller):
    def __init__(self):
        self.exchange_name = u'BITSTAMP_BTC_EUR'
        self.url = 'https://www.bitstamp.net/api/v2/order_book/btceur/'
        self.poll_time = 1

    # API response format:
    # {
    #   "asks":[["0.09022887",1704.11453071]],
    #   "bids":[["0.09000011",3.74072284]],
    #   "isFrozen":"0","seq":477056211,
    # }
    def parse_order(self, order):
        return [order[0], order[1], '']




import pyximport; pyximport.install()

from gryphon.lib.exchange.kraken_btc_eur import KrakenBTCEURExchange
from gryphon.tests.exceptional.exchange_wrappers.live_orders import LiveOrdersTest


class TestKrakenBTCEURLiveOrders(LiveOrdersTest):
    def __init__(self):
        # KrakenBTCEUR BTCEUR only allows resolution down to ten cents.
        self.order1_price_amount = '0.1'
        self.order2_price_amount = '0.2'
        self.sleep_time = 3  # KrakenBTCEUR is ridiculous about their rate limits.

    def setUp(self):
        self.exchange = KrakenBTCEURExchange()



"""
Some simple tests for the Multiexchange Linear Market Making strategy.
"""

import pyximport; pyximport.install()

import unittest

from gryphon.execution.strategies.builtin.multiexchange_linear import Multiexchange_linear
from gryphon.execution.harness.harness import Harness
from gryphon.lib.money import Money


class TestBitstampExchangeCoordinator(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_calculate_base_volumes_simple(self):
        """
        With a zero position, calculate_volumes should return the base_volume property
        of the strategy.
        """
        strat = Multiexchange_linear(None, None)

        strat._position = Money('0', 'BTC')

        bid_volume, ask_volume = strat.calculate_base_volumes()

        assert bid_volume == strat.base_volume
        assert ask_volume == strat.base_volume

    def test_calculate_base_volumes_harder(self):
        """
        When max_position > base_volume, calculate_volumes should return the minimum of
        the two.
        """
        strat = Multiexchange_linear(None, None)

        strat.max_position = Money('0.1', 'BTC')
        strat.base_volume = Money('0.2', 'BTC')
        strat._position = Money('0', 'BTC')

        bid_volume, ask_volume = strat.calculate_base_volumes()

        assert bid_volume == Money('0.1', 'BTC')
        assert ask_volume == Money('0.1', 'BTC')

    def test_calculate_base_volumes_hardest(self):
        """
        When we're near our maximum short position, we should place our base_volume
        amount on the bid and the order volume that would take us to our max position on
        the ask.
        """
        strat = Multiexchange_linear(None, None)

        strat.max_position = Money('0.1', 'BTC')
        strat.base_volume = Money('0.05', 'BTC')
        strat._position = Money('-0.09', 'BTC')

        bid_volume, ask_volume = strat.calculate_base_volumes()

        assert bid_volume == Money('0.05', 'BTC')
        assert ask_volume == Money('0.01', 'BTC')



"""
A collection of exceptions to handle regular failures from exchange APIs.
"""

class ExchangeException(Exception):
    pass

    
class CancelOrderNotFoundError(ExchangeException):
    pass


class InsufficientFundsError(ExchangeException):
    pass


class MinimumOrderSizeError(ExchangeException):
    pass


class NonceError(ExchangeException):
    pass


class NoEffectOrderCancelledError(ExchangeException):
    pass


class ExchangeAPIFailureException(ExchangeException):
    def __init__(self, exchange, response=None, message=''):
        if response != None and message:
            raise ValueError('Only specify one of response and message')

        if message:
            error_message = message
        else:
            error_message = 'No response' if response == None else response.text

        message = '[%s] %s' % (exchange.name, error_message)

        super(ExchangeAPIFailureException, self).__init__(message)
        self.exchange = exchange
        self.response = response


class ExchangeAPIErrorException(ExchangeException):
    def __init__(self, exchange, message):
        super_message = '[%s] %s' % (exchange.name, message)
        super(ExchangeAPIErrorException, self).__init__(super_message)
        self.exchange = exchange
        self.message = message


class CachedOrderbookFailure(ExchangeException):
    def __init__(self, exchange, message):
        super_message = '[%s] %s' % (exchange.name, message)
        super(CachedOrderbookFailure, self).__init__(super_message)


class ExchangeNotIntegratedError(ExchangeException):
    """
    This exception is used to indicate that the user is trying to trade on an exchange
    that Gryphon doesn't know about or hasn't been integrated.
    """
    def __init__(self, exchange_name):
        message = """Gryphon does not have an integration for the exchange named %s. Maybe you forgot to qualify the exchange name with its trading pair, like BITSTAMP_BTC_USD, or maybe Gryphon just doesn\'t support that exchange.""" % exchange_name

        super(ExchangeNotIntegratedError, self).__init__(message)

